import os
from typing import BinaryIO

from qwak.exceptions import QwakHTTPException

from .file_input import FileInput


class ImageInput(FileInput):
    def __init__(
        self,
        **base_kwargs,
    ):
        try:
            import imageio  # noqa: F401
        except ImportError:
            raise ImportError("imageio package is required to use ImageInput adapter")

        super().__init__(**base_kwargs)
        self.accept_image_formats = {".jpg", ".png", ".jpeg", ".tiff", ".webp", ".bmp"}

    def extract_user_func_args(self, data: BinaryIO) -> "numpy.ndarray":  # noqa F821
        try:
            import imageio
        except ImportError:
            raise ImportError("imageio package is required to use ImageInput adapter")

        if getattr(data, "name", None) and not self.check_file_extension(
            data.name, self.accept_image_formats
        ):
            raise QwakHTTPException(
                400,
                message=f"Current service only accepts {self.accept_image_formats} formats",
            )
        try:
            return imageio.imread(data, pilmode="RGB")
        except ValueError as e:
            raise QwakHTTPException(400, message=str(e))

    @staticmethod
    def check_file_extension(file_name, supported_types):
        if not file_name:
            return False
        _, extension = os.path.splitext(file_name)
        return extension.lower() in (supported_types or [])



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import qwak.fitness_service.constructs_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _PurchaseOption:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _PurchaseOptionEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_PurchaseOption.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID_PURCHASE_OPTION: _PurchaseOption.ValueType  # 0
    SPOT_PURCHASE_OPTION: _PurchaseOption.ValueType  # 1
    ONDEMAND_PURCHASE_OPTION: _PurchaseOption.ValueType  # 2

class PurchaseOption(_PurchaseOption, metaclass=_PurchaseOptionEnumTypeWrapper): ...

INVALID_PURCHASE_OPTION: PurchaseOption.ValueType  # 0
SPOT_PURCHASE_OPTION: PurchaseOption.ValueType  # 1
ONDEMAND_PURCHASE_OPTION: PurchaseOption.ValueType  # 2
global___PurchaseOption = PurchaseOption

class BuildSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUILD_PROPERTIES_FIELD_NUMBER: builtins.int
    BUILD_ENV_FIELD_NUMBER: builtins.int
    CPU_RESOURCES_FIELD_NUMBER: builtins.int
    GPU_RESOURCES_FIELD_NUMBER: builtins.int
    VERBOSE_FIELD_NUMBER: builtins.int
    BUILD_CONFIG_FIELD_NUMBER: builtins.int
    BUILD_V1_FLAG_FIELD_NUMBER: builtins.int
    BUILD_PROPERTIES_V1_FIELD_NUMBER: builtins.int
    PURCHASE_OPTION_FIELD_NUMBER: builtins.int
    BUILD_DESTINED_IMAGE_FIELD_NUMBER: builtins.int
    PROVISION_INSTANCE_TIMEOUT_FIELD_NUMBER: builtins.int
    @property
    def build_properties(self) -> qwak.fitness_service.constructs_pb2.BuildProperties: ...
    @property
    def build_env(self) -> qwak.fitness_service.constructs_pb2.BuildEnv: ...
    @property
    def cpu_resources(self) -> qwak.fitness_service.constructs_pb2.CpuResources:
        """CPU Properties"""
    @property
    def gpu_resources(self) -> qwak.fitness_service.constructs_pb2.GpuResources:
        """GPU Properties"""
    verbose: builtins.int
    """Logs verbose mode 0: warning, 1: info, 2: debug (by qwak.tools.logger.py VERBOSITY_LEVEL_MAPPING from qwak-sdk)"""
    build_config: builtins.str
    """replace the config passed as proto json format"""
    build_v1_flag: builtins.bool
    """Build v1 flag"""
    @property
    def build_properties_v1(self) -> global___BuildPropertiesV1:
        """Build Properties V1"""
    purchase_option: global___PurchaseOption.ValueType
    """Purchase Option"""
    build_destined_image: builtins.str
    """The image name to save build to"""
    provision_instance_timeout: builtins.int
    """The image name to save build to"""
    def __init__(
        self,
        *,
        build_properties: qwak.fitness_service.constructs_pb2.BuildProperties | None = ...,
        build_env: qwak.fitness_service.constructs_pb2.BuildEnv | None = ...,
        cpu_resources: qwak.fitness_service.constructs_pb2.CpuResources | None = ...,
        gpu_resources: qwak.fitness_service.constructs_pb2.GpuResources | None = ...,
        verbose: builtins.int = ...,
        build_config: builtins.str = ...,
        build_v1_flag: builtins.bool = ...,
        build_properties_v1: global___BuildPropertiesV1 | None = ...,
        purchase_option: global___PurchaseOption.ValueType = ...,
        build_destined_image: builtins.str = ...,
        provision_instance_timeout: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Resources", b"Resources", "build_env", b"build_env", "build_properties", b"build_properties", "build_properties_v1", b"build_properties_v1", "cpu_resources", b"cpu_resources", "gpu_resources", b"gpu_resources"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Resources", b"Resources", "build_config", b"build_config", "build_destined_image", b"build_destined_image", "build_env", b"build_env", "build_properties", b"build_properties", "build_properties_v1", b"build_properties_v1", "build_v1_flag", b"build_v1_flag", "cpu_resources", b"cpu_resources", "gpu_resources", b"gpu_resources", "provision_instance_timeout", b"provision_instance_timeout", "purchase_option", b"purchase_option", "verbose", b"verbose"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Resources", b"Resources"]) -> typing_extensions.Literal["cpu_resources", "gpu_resources"] | None: ...

global___BuildSpec = BuildSpec

class BuildPropertiesV1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUILD_CONFIG_URL_FIELD_NUMBER: builtins.int
    QWAK_SDK_WHEEL_URL_FIELD_NUMBER: builtins.int
    QWAK_SDK_VERSION_URL_FIELD_NUMBER: builtins.int
    build_config_url: builtins.str
    """Build V1 - build config url"""
    qwak_sdk_wheel_url: builtins.str
    """Build V1 - qwak-sdk wheel url"""
    qwak_sdk_version_url: builtins.str
    """Build V1 - qwak-sdk version"""
    def __init__(
        self,
        *,
        build_config_url: builtins.str = ...,
        qwak_sdk_wheel_url: builtins.str = ...,
        qwak_sdk_version_url: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["build_config_url", b"build_config_url", "qwak_sdk_version_url", b"qwak_sdk_version_url", "qwak_sdk_wheel_url", b"qwak_sdk_wheel_url"]) -> None: ...

global___BuildPropertiesV1 = BuildPropertiesV1



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/feature_store/features/deployment.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,qwak/feature_store/features/deployment.proto\x12\x1bqwak.feature.store.features\x1a\x1fgoogle/protobuf/timestamp.proto\"\x9d\x03\n\x1c\x44\x65ploymentUpdateRequestState\x12Y\n\x14invalid_request_type\x18\x01 \x01(\x0b\x32\x39.qwak.feature.store.features.DeploymentInvalidRequestTypeH\x00\x12W\n\x13update_request_type\x18\x02 \x01(\x0b\x32\x38.qwak.feature.store.features.DeploymentUpdateRequestTypeH\x00\x12[\n\x15undeploy_request_type\x18\x03 \x01(\x0b\x32:.qwak.feature.store.features.DeploymentUndeployRequestTypeH\x00\x12\x64\n deployment_update_request_status\x18\x04 \x01(\x0e\x32:.qwak.feature.store.features.DeploymentUpdateRequestStatusB\x06\n\x04type\"\x1e\n\x1c\x44\x65ploymentInvalidRequestType\"\x1d\n\x1b\x44\x65ploymentUpdateRequestType\"\x1f\n\x1d\x44\x65ploymentUndeployRequestType\"\xce\x01\n\x0f\x44\x65ploymentState\x12Z\n\x14\x65xtractor_identifier\x18\x01 \x01(\x0b\x32:.qwak.feature.store.features.ExtractorDeploymentIdentifierH\x00\x12L\n\x0estatus_details\x18\x05 \x01(\x0b\x32\x34.qwak.feature.store.features.DeploymentStatusDetailsB\x11\n\x0fidentifier_type\"m\n\x1d\x45xtractorDeploymentIdentifier\x12\x1f\n\x17\x65xtractor_deployment_id\x18\x01 \x01(\t\x12\x14\n\x0c\x65xtractor_id\x18\x02 \x01(\t\x12\x15\n\rfeatureset_id\x18\x03 \x01(\t\"\xbc\x02\n\x17\x44\x65ploymentStatusDetails\x12g\n!update_in_progress_status_details\x18\x01 \x01(\x0b\x32:.qwak.feature.store.features.UpdateInProgressStatusDetailsH\x00\x12S\n\x16running_status_details\x18\x02 \x01(\x0b\x32\x31.qwak.feature.store.features.RunningStatusDetailsH\x00\x12Q\n\x15\x66\x61iled_status_details\x18\x03 \x01(\x0b\x32\x30.qwak.feature.store.features.FailedStatusDetailsH\x00\x42\x10\n\x0estatus_details\"\xa7\x01\n\x1dUpdateInProgressStatusDetails\x12\x42\n\x0cold_instance\x18\x01 \x01(\x0b\x32,.qwak.feature.store.features.InstanceDetails\x12\x42\n\x0cnew_instance\x18\x02 \x01(\x0b\x32,.qwak.feature.store.features.InstanceDetails\"^\n\x14RunningStatusDetails\x12\x46\n\x10running_instance\x18\x01 \x01(\x0b\x32,.qwak.feature.store.features.InstanceDetails\"\xe6\x01\n\x13\x46\x61iledStatusDetails\x12\x44\n\x0f\x66\x61ilure_details\x18\x01 \x01(\x0b\x32+.qwak.feature.store.features.FailureDetails\x12\x42\n\x0cold_instance\x18\x02 \x01(\x0b\x32,.qwak.feature.store.features.InstanceDetails\x12\x45\n\x0f\x66\x61iled_instance\x18\x03 \x01(\x0b\x32,.qwak.feature.store.features.InstanceDetails\"\x92\x01\n\x0e\x46\x61ilureDetails\x12`\n\x1e\x64\x65ployment_failure_reason_code\x18\x01 \x01(\x0e\x32\x38.qwak.feature.store.features.DeploymentFailureReasonCode\x12\x1e\n\x16\x66\x61ilure_reason_details\x18\x02 \x01(\t\"\x9a\x01\n\x0fInstanceDetails\x12X\n\x14\x65xtractor_identifier\x18\x01 \x01(\x0b\x32\x38.qwak.feature.store.features.ExtractorInstanceIdentifierH\x00\x12\x1a\n\x12\x61vailable_replicas\x18\x04 \x01(\x05\x42\x11\n\x0fidentifier_type\"M\n\x1b\x45xtractorInstanceIdentifier\x12.\n&extractor_deployment_update_request_id\x18\x01 \x01(\t*\xa0\x02\n\x1d\x44\x65ploymentUpdateRequestStatus\x12,\n(DEPLOYMENT_UPDATE_REQUEST_INVALID_STATUS\x10\x00\x12%\n!INVALID_DEPLOYMENT_UPDATE_REQUEST\x10\x01\x12(\n$INITIATING_DEPLOYMENT_UPDATE_REQUEST\x10\x02\x12/\n+FAILED_INITIATING_DEPLOYMENT_UPDATE_REQUEST\x10\x03\x12%\n!PENDING_DEPLOYMENT_UPDATE_REQUEST\x10\x04\x12(\n$SUCCESSFUL_DEPLOYMENT_UPDATE_REQUEST\x10\x05*\x9e\x01\n\x10\x44\x65ploymentStatus\x12\x1d\n\x19\x44\x45PLOYMENT_INVALID_STATUS\x10\x00\x12\x16\n\x12\x44\x45PLOYMENT_RUNNING\x10\x01\x12!\n\x1d\x44\x45PLOYMENT_UPDATE_IN_PROGRESS\x10\x02\x12\x19\n\x15\x44\x45PLOYMENT_UNDEPLOYED\x10\x03\x12\x15\n\x11\x44\x45PLOYMENT_FAILED\x10\x04*\x8d\x02\n\x1b\x44\x65ploymentFailureReasonCode\x12\x18\n\x14INVALID_FAILURE_CODE\x10\x00\x12\x12\n\x0eUNKNOWN_REASON\x10\x01\x12\x1e\n\x1aNO_REPLICA_SETS_PODS_FOUND\x10\x02\x12\x17\n\x13\x43ONTAINER_NOT_FOUND\x10\x03\x12\x11\n\rUNSCHEDULABLE\x10\x04\x12\x1a\n\x16\x44OCKER_IMAGE_NOT_FOUND\x10\x05\x12\x19\n\x15MEMORY_LIMIT_EXCEEDED\x10\x06\x12\x0e\n\nCRASH_LOOP\x10\x07\x12-\n)CONTAINER_FAIL_TO_LOAD_FOR_UNKNOWN_REASON\x10\x08\x42[\n&com.qwak.ai.feature.store.features.apiP\x01Z/qwak/featurestore/features;featurestorefeaturesb\x06proto3')

_DEPLOYMENTUPDATEREQUESTSTATUS = DESCRIPTOR.enum_types_by_name['DeploymentUpdateRequestStatus']
DeploymentUpdateRequestStatus = enum_type_wrapper.EnumTypeWrapper(_DEPLOYMENTUPDATEREQUESTSTATUS)
_DEPLOYMENTSTATUS = DESCRIPTOR.enum_types_by_name['DeploymentStatus']
DeploymentStatus = enum_type_wrapper.EnumTypeWrapper(_DEPLOYMENTSTATUS)
_DEPLOYMENTFAILUREREASONCODE = DESCRIPTOR.enum_types_by_name['DeploymentFailureReasonCode']
DeploymentFailureReasonCode = enum_type_wrapper.EnumTypeWrapper(_DEPLOYMENTFAILUREREASONCODE)
DEPLOYMENT_UPDATE_REQUEST_INVALID_STATUS = 0
INVALID_DEPLOYMENT_UPDATE_REQUEST = 1
INITIATING_DEPLOYMENT_UPDATE_REQUEST = 2
FAILED_INITIATING_DEPLOYMENT_UPDATE_REQUEST = 3
PENDING_DEPLOYMENT_UPDATE_REQUEST = 4
SUCCESSFUL_DEPLOYMENT_UPDATE_REQUEST = 5
DEPLOYMENT_INVALID_STATUS = 0
DEPLOYMENT_RUNNING = 1
DEPLOYMENT_UPDATE_IN_PROGRESS = 2
DEPLOYMENT_UNDEPLOYED = 3
DEPLOYMENT_FAILED = 4
INVALID_FAILURE_CODE = 0
UNKNOWN_REASON = 1
NO_REPLICA_SETS_PODS_FOUND = 2
CONTAINER_NOT_FOUND = 3
UNSCHEDULABLE = 4
DOCKER_IMAGE_NOT_FOUND = 5
MEMORY_LIMIT_EXCEEDED = 6
CRASH_LOOP = 7
CONTAINER_FAIL_TO_LOAD_FOR_UNKNOWN_REASON = 8


_DEPLOYMENTUPDATEREQUESTSTATE = DESCRIPTOR.message_types_by_name['DeploymentUpdateRequestState']
_DEPLOYMENTINVALIDREQUESTTYPE = DESCRIPTOR.message_types_by_name['DeploymentInvalidRequestType']
_DEPLOYMENTUPDATEREQUESTTYPE = DESCRIPTOR.message_types_by_name['DeploymentUpdateRequestType']
_DEPLOYMENTUNDEPLOYREQUESTTYPE = DESCRIPTOR.message_types_by_name['DeploymentUndeployRequestType']
_DEPLOYMENTSTATE = DESCRIPTOR.message_types_by_name['DeploymentState']
_EXTRACTORDEPLOYMENTIDENTIFIER = DESCRIPTOR.message_types_by_name['ExtractorDeploymentIdentifier']
_DEPLOYMENTSTATUSDETAILS = DESCRIPTOR.message_types_by_name['DeploymentStatusDetails']
_UPDATEINPROGRESSSTATUSDETAILS = DESCRIPTOR.message_types_by_name['UpdateInProgressStatusDetails']
_RUNNINGSTATUSDETAILS = DESCRIPTOR.message_types_by_name['RunningStatusDetails']
_FAILEDSTATUSDETAILS = DESCRIPTOR.message_types_by_name['FailedStatusDetails']
_FAILUREDETAILS = DESCRIPTOR.message_types_by_name['FailureDetails']
_INSTANCEDETAILS = DESCRIPTOR.message_types_by_name['InstanceDetails']
_EXTRACTORINSTANCEIDENTIFIER = DESCRIPTOR.message_types_by_name['ExtractorInstanceIdentifier']
DeploymentUpdateRequestState = _reflection.GeneratedProtocolMessageType('DeploymentUpdateRequestState', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTUPDATEREQUESTSTATE,
  '__module__' : 'qwak.feature_store.features.deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.DeploymentUpdateRequestState)
  })
_sym_db.RegisterMessage(DeploymentUpdateRequestState)

DeploymentInvalidRequestType = _reflection.GeneratedProtocolMessageType('DeploymentInvalidRequestType', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTINVALIDREQUESTTYPE,
  '__module__' : 'qwak.feature_store.features.deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.DeploymentInvalidRequestType)
  })
_sym_db.RegisterMessage(DeploymentInvalidRequestType)

DeploymentUpdateRequestType = _reflection.GeneratedProtocolMessageType('DeploymentUpdateRequestType', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTUPDATEREQUESTTYPE,
  '__module__' : 'qwak.feature_store.features.deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.DeploymentUpdateRequestType)
  })
_sym_db.RegisterMessage(DeploymentUpdateRequestType)

DeploymentUndeployRequestType = _reflection.GeneratedProtocolMessageType('DeploymentUndeployRequestType', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTUNDEPLOYREQUESTTYPE,
  '__module__' : 'qwak.feature_store.features.deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.DeploymentUndeployRequestType)
  })
_sym_db.RegisterMessage(DeploymentUndeployRequestType)

DeploymentState = _reflection.GeneratedProtocolMessageType('DeploymentState', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTSTATE,
  '__module__' : 'qwak.feature_store.features.deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.DeploymentState)
  })
_sym_db.RegisterMessage(DeploymentState)

ExtractorDeploymentIdentifier = _reflection.GeneratedProtocolMessageType('ExtractorDeploymentIdentifier', (_message.Message,), {
  'DESCRIPTOR' : _EXTRACTORDEPLOYMENTIDENTIFIER,
  '__module__' : 'qwak.feature_store.features.deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.ExtractorDeploymentIdentifier)
  })
_sym_db.RegisterMessage(ExtractorDeploymentIdentifier)

DeploymentStatusDetails = _reflection.GeneratedProtocolMessageType('DeploymentStatusDetails', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTSTATUSDETAILS,
  '__module__' : 'qwak.feature_store.features.deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.DeploymentStatusDetails)
  })
_sym_db.RegisterMessage(DeploymentStatusDetails)

UpdateInProgressStatusDetails = _reflection.GeneratedProtocolMessageType('UpdateInProgressStatusDetails', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEINPROGRESSSTATUSDETAILS,
  '__module__' : 'qwak.feature_store.features.deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.UpdateInProgressStatusDetails)
  })
_sym_db.RegisterMessage(UpdateInProgressStatusDetails)

RunningStatusDetails = _reflection.GeneratedProtocolMessageType('RunningStatusDetails', (_message.Message,), {
  'DESCRIPTOR' : _RUNNINGSTATUSDETAILS,
  '__module__' : 'qwak.feature_store.features.deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.RunningStatusDetails)
  })
_sym_db.RegisterMessage(RunningStatusDetails)

FailedStatusDetails = _reflection.GeneratedProtocolMessageType('FailedStatusDetails', (_message.Message,), {
  'DESCRIPTOR' : _FAILEDSTATUSDETAILS,
  '__module__' : 'qwak.feature_store.features.deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.FailedStatusDetails)
  })
_sym_db.RegisterMessage(FailedStatusDetails)

FailureDetails = _reflection.GeneratedProtocolMessageType('FailureDetails', (_message.Message,), {
  'DESCRIPTOR' : _FAILUREDETAILS,
  '__module__' : 'qwak.feature_store.features.deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.FailureDetails)
  })
_sym_db.RegisterMessage(FailureDetails)

InstanceDetails = _reflection.GeneratedProtocolMessageType('InstanceDetails', (_message.Message,), {
  'DESCRIPTOR' : _INSTANCEDETAILS,
  '__module__' : 'qwak.feature_store.features.deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.InstanceDetails)
  })
_sym_db.RegisterMessage(InstanceDetails)

ExtractorInstanceIdentifier = _reflection.GeneratedProtocolMessageType('ExtractorInstanceIdentifier', (_message.Message,), {
  'DESCRIPTOR' : _EXTRACTORINSTANCEIDENTIFIER,
  '__module__' : 'qwak.feature_store.features.deployment_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.ExtractorInstanceIdentifier)
  })
_sym_db.RegisterMessage(ExtractorInstanceIdentifier)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n&com.qwak.ai.feature.store.features.apiP\001Z/qwak/featurestore/features;featurestorefeatures'
  _DEPLOYMENTUPDATEREQUESTSTATUS._serialized_start=2146
  _DEPLOYMENTUPDATEREQUESTSTATUS._serialized_end=2434
  _DEPLOYMENTSTATUS._serialized_start=2437
  _DEPLOYMENTSTATUS._serialized_end=2595
  _DEPLOYMENTFAILUREREASONCODE._serialized_start=2598
  _DEPLOYMENTFAILUREREASONCODE._serialized_end=2867
  _DEPLOYMENTUPDATEREQUESTSTATE._serialized_start=111
  _DEPLOYMENTUPDATEREQUESTSTATE._serialized_end=524
  _DEPLOYMENTINVALIDREQUESTTYPE._serialized_start=526
  _DEPLOYMENTINVALIDREQUESTTYPE._serialized_end=556
  _DEPLOYMENTUPDATEREQUESTTYPE._serialized_start=558
  _DEPLOYMENTUPDATEREQUESTTYPE._serialized_end=587
  _DEPLOYMENTUNDEPLOYREQUESTTYPE._serialized_start=589
  _DEPLOYMENTUNDEPLOYREQUESTTYPE._serialized_end=620
  _DEPLOYMENTSTATE._serialized_start=623
  _DEPLOYMENTSTATE._serialized_end=829
  _EXTRACTORDEPLOYMENTIDENTIFIER._serialized_start=831
  _EXTRACTORDEPLOYMENTIDENTIFIER._serialized_end=940
  _DEPLOYMENTSTATUSDETAILS._serialized_start=943
  _DEPLOYMENTSTATUSDETAILS._serialized_end=1259
  _UPDATEINPROGRESSSTATUSDETAILS._serialized_start=1262
  _UPDATEINPROGRESSSTATUSDETAILS._serialized_end=1429
  _RUNNINGSTATUSDETAILS._serialized_start=1431
  _RUNNINGSTATUSDETAILS._serialized_end=1525
  _FAILEDSTATUSDETAILS._serialized_start=1528
  _FAILEDSTATUSDETAILS._serialized_end=1758
  _FAILUREDETAILS._serialized_start=1761
  _FAILUREDETAILS._serialized_end=1907
  _INSTANCEDETAILS._serialized_start=1910
  _INSTANCEDETAILS._serialized_end=2064
  _EXTRACTORINSTANCEIDENTIFIER._serialized_start=2066
  _EXTRACTORINSTANCEIDENTIFIER._serialized_end=2143
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/models/models.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from _qwak_proto.qwak.builds import builds_pb2 as qwak_dot_builds_dot_builds__pb2
from _qwak_proto.qwak.audience.v1 import audience_pb2 as qwak_dot_audience_dot_v1_dot_audience__pb2
from _qwak_proto.qwak.deployment import deployment_pb2 as qwak_dot_deployment_dot_deployment__pb2
from _qwak_proto.qwak.build.v1 import build_pb2 as qwak_dot_build_dot_v1_dot_build__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18qwak/models/models.proto\x12\x16qwak.models.management\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x18qwak/builds/builds.proto\x1a\x1fqwak/audience/v1/audience.proto\x1a qwak/deployment/deployment.proto\x1a\x19qwak/build/v1/build.proto\"K\n\x12\x43reateModelRequest\x12\x35\n\nmodel_spec\x18\x01 \x01(\x0b\x32!.qwak.models.management.ModelSpec\"\'\n\x13\x43reateModelResponse\x12\x10\n\x08model_id\x18\x01 \x01(\t\"\'\n\x11ListModelsRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\"C\n\x12ListModelsResponse\x12-\n\x06models\x18\x01 \x03(\x0b\x32\x1d.qwak.models.management.Model\"I\n\x0fGetModelRequest\x12\x12\n\x08model_id\x18\x01 \x01(\tH\x00\x12\x14\n\nmodel_uuid\x18\x02 \x01(\tH\x00\x42\x0c\n\nidentifier\"@\n\x10GetModelResponse\x12,\n\x05model\x18\x01 \x01(\x0b\x32\x1d.qwak.models.management.Model\":\n\x12\x44\x65leteModelRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\"\x15\n\x13\x44\x65leteModelResponse\"q\n\x12UpdateModelRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12I\n\x11model_spec_config\x18\x02 \x01(\x0b\x32..qwak.models.management.ModelSpecConfiguration\"L\n\x13UpdateModelResponse\x12\x35\n\nmodel_spec\x18\x01 \x01(\x0b\x32!.qwak.models.management.ModelSpec\"<\n\x13\x43reateBranchRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x13\n\x0b\x62ranch_name\x18\x02 \x01(\t\")\n\x14\x43reateBranchResponse\x12\x11\n\tbranch_id\x18\x01 \x01(\t\"(\n\x13\x44\x65leteBranchRequest\x12\x11\n\tbranch_id\x18\x01 \x01(\t\"\x16\n\x14\x44\x65leteBranchResponse\"N\n\x14IsModelExistsRequest\x12\x12\n\x08model_id\x18\x01 \x01(\tH\x00\x12\x14\n\nmodel_uuid\x18\x02 \x01(\tH\x00\x42\x0c\n\nidentifier\"\'\n\x15IsModelExistsResponse\x12\x0e\n\x06\x65xists\x18\x01 \x01(\x08\"\xda\x01\n\tModelSpec\x12\x16\n\x0emodel_named_id\x18\x01 \x01(\t\x12\x14\n\x0c\x64isplay_name\x18\x02 \x01(\t\x12\x14\n\nproject_id\x18\x03 \x01(\tH\x00\x12\x16\n\x0cproject_name\x18\x06 \x01(\tH\x00\x12\x19\n\x11model_description\x18\x04 \x01(\t\x12@\n\x10model_build_type\x18\x05 \x01(\x0b\x32&.qwak.models.management.ModelBuildTypeB\x14\n\x12project_identifier\"\xc7\x02\n\nModelBrief\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x14\n\x0c\x64isplay_name\x18\x02 \x01(\t\x12.\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x34\n\x10last_modified_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10last_modified_by\x18\x05 \x01(\t\x12\x12\n\ncreated_by\x18\x06 \x01(\t\x12\x19\n\x11model_description\x18\x08 \x01(\t\x12\x12\n\nproject_id\x18\t \x01(\t\x12N\n\x17\x64\x65ployment_model_status\x18\n \x01(\x0e\x32-.qwak.models.management.DeploymentModelStatus\"3\n\x16ModelSpecConfiguration\x12\x19\n\x11model_description\x18\x04 \x01(\t\"\xd9\x04\n\x05Model\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x14\n\x0c\x64isplay_name\x18\x02 \x01(\t\x12.\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x34\n\x10last_modified_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10last_modified_by\x18\x05 \x01(\t\x12\x12\n\ncreated_by\x18\x06 \x01(\t\x12\x39\n\x0cmodel_status\x18\x07 \x01(\x0e\x32#.qwak.models.management.ModelStatus\x12\x19\n\x11model_description\x18\x08 \x01(\t\x12\x12\n\nproject_id\x18\t \x01(\t\x12\x30\n\x08\x62ranches\x18\n \x03(\x0b\x32\x1e.qwak.models.management.Branch\x12N\n\x17\x64\x65ployment_model_status\x18\x0b \x01(\x0e\x32-.qwak.models.management.DeploymentModelStatus\x12\x0c\n\x04uuid\x18\x0c \x01(\t\x12\x42\n\nmodel_type\x18\r \x01(\x0e\x32..qwak.deployment.management.KubeDeploymentType\x12\x14\n\x0cproject_name\x18\x0e \x01(\t\x12@\n\x10model_build_type\x18\x0f \x01(\x0b\x32&.qwak.models.management.ModelBuildType\"\x96\x01\n\x0eModelBuildType\x12\x30\n\x06source\x18\x01 \x01(\x0b\x32\x1e.qwak.models.management.SourceH\x00\x12J\n\x14model_registry_image\x18\x02 \x01(\x0b\x32*.qwak.models.management.ModelRegistryImageH\x00\x42\x06\n\x04type\"\x08\n\x06Source\"3\n\x12ModelRegistryImage\x12\x1d\n\x15last_image_identifier\x18\x01 \x01(\t\"\xeb\x01\n\x06\x42ranch\x12\x11\n\tbranch_id\x18\x01 \x01(\t\x12\x13\n\x0b\x62ranch_name\x18\x02 \x01(\t\x12\x42\n\rbranch_status\x18\x04 \x01(\x0e\x32+.qwak.models.management.Branch.BranchStatus\x12\x10\n\x08model_id\x18\x05 \x01(\t\x12\x1d\n\x11\x64\x65ployed_build_id\x18\x06 \x01(\tB\x02\x18\x01\"D\n\x0c\x42ranchStatus\x12\x0b\n\x07INVALID\x10\x00\x12\n\n\x06\x41\x43TIVE\x10\x03\x12\r\n\tSUSPENDED\x10\x02\x12\x0c\n\x08\x44ISABLED\x10\x01\"B\n\x1bGetDeploymentDetailsRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x11\n\tbranch_id\x18\x02 \x01(\t\"e\n\x1cGetDeploymentDetailsResponse\x12\x45\n\x12\x64\x65ployment_details\x18\x01 \x01(\x0b\x32).qwak.models.management.DeploymentDetails\"\x93\x02\n\x11\x44\x65ploymentDetails\x12\x31\n\rlast_deployed\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08\x62uild_id\x18\x02 \x01(\t\x12\x16\n\x0enumber_of_pods\x18\x03 \x01(\x05\x12\x14\n\x0c\x63pu_fraction\x18\x04 \x01(\x02\x12\x15\n\rmemory_amount\x18\x05 \x01(\x05\x12\x38\n\x0cmemory_units\x18\x06 \x01(\x0e\x32\".qwak.models.management.MemoryUnit\x12\x19\n\x11number_of_workers\x18\x07 \x01(\x05\x12\x1f\n\x17http_request_timeout_ms\x18\x08 \x01(\x05\"/\n\x19ListModelsMetadataRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\"[\n\x1aListModelsMetadataResponse\x12=\n\x0emodel_metadata\x18\x01 \x03(\x0b\x32%.qwak.models.management.ModelMetadata\"+\n\x17GetModelMetadataRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\"Y\n\x18GetModelMetadataResponse\x12=\n\x0emodel_metadata\x18\x01 \x01(\x0b\x32%.qwak.models.management.ModelMetadata\"h\n\x11\x45nvironmentRoutes\x12\x13\n\x0b\x65nvironment\x18\x01 \x01(\t\x12>\n\x0f\x61udience_routes\x18\x02 \x03(\x0b\x32%.qwak.audience.v1.AudienceRoutesEntry\"\xb8\x05\n\rModelMetadata\x12,\n\x05model\x18\x01 \x01(\x0b\x32\x1d.qwak.models.management.Model\x12I\n\x12\x64\x65ployment_details\x18\x02 \x03(\x0b\x32-.qwak.deployment.management.DeploymentDetails\x12r\n\x1e\x61udience_routes_by_environment\x18\x03 \x03(\x0b\x32\x46.qwak.models.management.ModelMetadata.AudienceRoutesByEnvironmentEntryB\x02\x18\x01\x12\x30\n\x05\x62uild\x18\x04 \x03(\x0b\x32\x1d.qwak.builds.management.BuildB\x02\x18\x01\x12}\n&audience_routes_grouped_by_environment\x18\x05 \x03(\x0b\x32M.qwak.models.management.ModelMetadata.AudienceRoutesGroupedByEnvironmentEntry\x12(\n\x06\x62uilds\x18\x06 \x03(\x0b\x32\x18.com.qwak.build.v1.Build\x1ai\n AudienceRoutesByEnvironmentEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32%.qwak.audience.v1.AudienceRoutesEntry:\x02\x38\x01\x1at\n\'AudienceRoutesGroupedByEnvironmentEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x38\n\x05value\x18\x02 \x01(\x0b\x32).qwak.models.management.EnvironmentRoutes:\x02\x38\x01*O\n\x15\x44\x65ploymentModelStatus\x12\x1d\n\x19\x44\x45PLOYMENT_STATUS_INVALID\x10\x00\x12\n\n\x06ONLINE\x10\x01\x12\x0b\n\x07OFFLINE\x10\x02*C\n\x0bModelStatus\x12\x0b\n\x07INVALID\x10\x00\x12\n\n\x06\x41\x43TIVE\x10\x03\x12\r\n\tSUSPENDED\x10\x02\x12\x0c\n\x08\x44ISABLED\x10\x01*7\n\nMemoryUnit\x12\x17\n\x13MEMORY_UNIT_UNKNOWN\x10\x00\x12\x07\n\x03MIB\x10\x01\x12\x07\n\x03GIB\x10\x02\x32\xe0\t\n\x17ModelsManagementService\x12\x66\n\x0b\x43reateModel\x12*.qwak.models.management.CreateModelRequest\x1a+.qwak.models.management.CreateModelResponse\x12\x63\n\nListModels\x12).qwak.models.management.ListModelsRequest\x1a*.qwak.models.management.ListModelsResponse\x12]\n\x08GetModel\x12\'.qwak.models.management.GetModelRequest\x1a(.qwak.models.management.GetModelResponse\x12\x66\n\x0b\x44\x65leteModel\x12*.qwak.models.management.DeleteModelRequest\x1a+.qwak.models.management.DeleteModelResponse\x12\x66\n\x0bUpdateModel\x12*.qwak.models.management.UpdateModelRequest\x1a+.qwak.models.management.UpdateModelResponse\x12n\n\x0c\x43reateBranch\x12+.qwak.models.management.CreateBranchRequest\x1a,.qwak.models.management.CreateBranchResponse\"\x03\x88\x02\x01\x12n\n\x0c\x44\x65leteBranch\x12+.qwak.models.management.DeleteBranchRequest\x1a,.qwak.models.management.DeleteBranchResponse\"\x03\x88\x02\x01\x12\x86\x01\n\x14GetDeploymentDetails\x12\x33.qwak.models.management.GetDeploymentDetailsRequest\x1a\x34.qwak.models.management.GetDeploymentDetailsResponse\"\x03\x88\x02\x01\x12l\n\rIsModelExists\x12,.qwak.models.management.IsModelExistsRequest\x1a-.qwak.models.management.IsModelExistsResponse\x12{\n\x12ListModelsMetadata\x12\x31.qwak.models.management.ListModelsMetadataRequest\x1a\x32.qwak.models.management.ListModelsMetadataResponse\x12u\n\x10GetModelMetadata\x12/.qwak.models.management.GetModelMetadataRequest\x1a\x30.qwak.models.management.GetModelMetadataResponseB%\n!com.qwak.ai.management.models.apiP\x01\x62\x06proto3')

_DEPLOYMENTMODELSTATUS = DESCRIPTOR.enum_types_by_name['DeploymentModelStatus']
DeploymentModelStatus = enum_type_wrapper.EnumTypeWrapper(_DEPLOYMENTMODELSTATUS)
_MODELSTATUS = DESCRIPTOR.enum_types_by_name['ModelStatus']
ModelStatus = enum_type_wrapper.EnumTypeWrapper(_MODELSTATUS)
_MEMORYUNIT = DESCRIPTOR.enum_types_by_name['MemoryUnit']
MemoryUnit = enum_type_wrapper.EnumTypeWrapper(_MEMORYUNIT)
DEPLOYMENT_STATUS_INVALID = 0
ONLINE = 1
OFFLINE = 2
INVALID = 0
ACTIVE = 3
SUSPENDED = 2
DISABLED = 1
MEMORY_UNIT_UNKNOWN = 0
MIB = 1
GIB = 2


_CREATEMODELREQUEST = DESCRIPTOR.message_types_by_name['CreateModelRequest']
_CREATEMODELRESPONSE = DESCRIPTOR.message_types_by_name['CreateModelResponse']
_LISTMODELSREQUEST = DESCRIPTOR.message_types_by_name['ListModelsRequest']
_LISTMODELSRESPONSE = DESCRIPTOR.message_types_by_name['ListModelsResponse']
_GETMODELREQUEST = DESCRIPTOR.message_types_by_name['GetModelRequest']
_GETMODELRESPONSE = DESCRIPTOR.message_types_by_name['GetModelResponse']
_DELETEMODELREQUEST = DESCRIPTOR.message_types_by_name['DeleteModelRequest']
_DELETEMODELRESPONSE = DESCRIPTOR.message_types_by_name['DeleteModelResponse']
_UPDATEMODELREQUEST = DESCRIPTOR.message_types_by_name['UpdateModelRequest']
_UPDATEMODELRESPONSE = DESCRIPTOR.message_types_by_name['UpdateModelResponse']
_CREATEBRANCHREQUEST = DESCRIPTOR.message_types_by_name['CreateBranchRequest']
_CREATEBRANCHRESPONSE = DESCRIPTOR.message_types_by_name['CreateBranchResponse']
_DELETEBRANCHREQUEST = DESCRIPTOR.message_types_by_name['DeleteBranchRequest']
_DELETEBRANCHRESPONSE = DESCRIPTOR.message_types_by_name['DeleteBranchResponse']
_ISMODELEXISTSREQUEST = DESCRIPTOR.message_types_by_name['IsModelExistsRequest']
_ISMODELEXISTSRESPONSE = DESCRIPTOR.message_types_by_name['IsModelExistsResponse']
_MODELSPEC = DESCRIPTOR.message_types_by_name['ModelSpec']
_MODELBRIEF = DESCRIPTOR.message_types_by_name['ModelBrief']
_MODELSPECCONFIGURATION = DESCRIPTOR.message_types_by_name['ModelSpecConfiguration']
_MODEL = DESCRIPTOR.message_types_by_name['Model']
_MODELBUILDTYPE = DESCRIPTOR.message_types_by_name['ModelBuildType']
_SOURCE = DESCRIPTOR.message_types_by_name['Source']
_MODELREGISTRYIMAGE = DESCRIPTOR.message_types_by_name['ModelRegistryImage']
_BRANCH = DESCRIPTOR.message_types_by_name['Branch']
_GETDEPLOYMENTDETAILSREQUEST = DESCRIPTOR.message_types_by_name['GetDeploymentDetailsRequest']
_GETDEPLOYMENTDETAILSRESPONSE = DESCRIPTOR.message_types_by_name['GetDeploymentDetailsResponse']
_DEPLOYMENTDETAILS = DESCRIPTOR.message_types_by_name['DeploymentDetails']
_LISTMODELSMETADATAREQUEST = DESCRIPTOR.message_types_by_name['ListModelsMetadataRequest']
_LISTMODELSMETADATARESPONSE = DESCRIPTOR.message_types_by_name['ListModelsMetadataResponse']
_GETMODELMETADATAREQUEST = DESCRIPTOR.message_types_by_name['GetModelMetadataRequest']
_GETMODELMETADATARESPONSE = DESCRIPTOR.message_types_by_name['GetModelMetadataResponse']
_ENVIRONMENTROUTES = DESCRIPTOR.message_types_by_name['EnvironmentRoutes']
_MODELMETADATA = DESCRIPTOR.message_types_by_name['ModelMetadata']
_MODELMETADATA_AUDIENCEROUTESBYENVIRONMENTENTRY = _MODELMETADATA.nested_types_by_name['AudienceRoutesByEnvironmentEntry']
_MODELMETADATA_AUDIENCEROUTESGROUPEDBYENVIRONMENTENTRY = _MODELMETADATA.nested_types_by_name['AudienceRoutesGroupedByEnvironmentEntry']
_BRANCH_BRANCHSTATUS = _BRANCH.enum_types_by_name['BranchStatus']
CreateModelRequest = _reflection.GeneratedProtocolMessageType('CreateModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEMODELREQUEST,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.CreateModelRequest)
  })
_sym_db.RegisterMessage(CreateModelRequest)

CreateModelResponse = _reflection.GeneratedProtocolMessageType('CreateModelResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEMODELRESPONSE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.CreateModelResponse)
  })
_sym_db.RegisterMessage(CreateModelResponse)

ListModelsRequest = _reflection.GeneratedProtocolMessageType('ListModelsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTMODELSREQUEST,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.ListModelsRequest)
  })
_sym_db.RegisterMessage(ListModelsRequest)

ListModelsResponse = _reflection.GeneratedProtocolMessageType('ListModelsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTMODELSRESPONSE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.ListModelsResponse)
  })
_sym_db.RegisterMessage(ListModelsResponse)

GetModelRequest = _reflection.GeneratedProtocolMessageType('GetModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETMODELREQUEST,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.GetModelRequest)
  })
_sym_db.RegisterMessage(GetModelRequest)

GetModelResponse = _reflection.GeneratedProtocolMessageType('GetModelResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETMODELRESPONSE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.GetModelResponse)
  })
_sym_db.RegisterMessage(GetModelResponse)

DeleteModelRequest = _reflection.GeneratedProtocolMessageType('DeleteModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEMODELREQUEST,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.DeleteModelRequest)
  })
_sym_db.RegisterMessage(DeleteModelRequest)

DeleteModelResponse = _reflection.GeneratedProtocolMessageType('DeleteModelResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEMODELRESPONSE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.DeleteModelResponse)
  })
_sym_db.RegisterMessage(DeleteModelResponse)

UpdateModelRequest = _reflection.GeneratedProtocolMessageType('UpdateModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEMODELREQUEST,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.UpdateModelRequest)
  })
_sym_db.RegisterMessage(UpdateModelRequest)

UpdateModelResponse = _reflection.GeneratedProtocolMessageType('UpdateModelResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEMODELRESPONSE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.UpdateModelResponse)
  })
_sym_db.RegisterMessage(UpdateModelResponse)

CreateBranchRequest = _reflection.GeneratedProtocolMessageType('CreateBranchRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEBRANCHREQUEST,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.CreateBranchRequest)
  })
_sym_db.RegisterMessage(CreateBranchRequest)

CreateBranchResponse = _reflection.GeneratedProtocolMessageType('CreateBranchResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEBRANCHRESPONSE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.CreateBranchResponse)
  })
_sym_db.RegisterMessage(CreateBranchResponse)

DeleteBranchRequest = _reflection.GeneratedProtocolMessageType('DeleteBranchRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEBRANCHREQUEST,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.DeleteBranchRequest)
  })
_sym_db.RegisterMessage(DeleteBranchRequest)

DeleteBranchResponse = _reflection.GeneratedProtocolMessageType('DeleteBranchResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEBRANCHRESPONSE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.DeleteBranchResponse)
  })
_sym_db.RegisterMessage(DeleteBranchResponse)

IsModelExistsRequest = _reflection.GeneratedProtocolMessageType('IsModelExistsRequest', (_message.Message,), {
  'DESCRIPTOR' : _ISMODELEXISTSREQUEST,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.IsModelExistsRequest)
  })
_sym_db.RegisterMessage(IsModelExistsRequest)

IsModelExistsResponse = _reflection.GeneratedProtocolMessageType('IsModelExistsResponse', (_message.Message,), {
  'DESCRIPTOR' : _ISMODELEXISTSRESPONSE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.IsModelExistsResponse)
  })
_sym_db.RegisterMessage(IsModelExistsResponse)

ModelSpec = _reflection.GeneratedProtocolMessageType('ModelSpec', (_message.Message,), {
  'DESCRIPTOR' : _MODELSPEC,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.ModelSpec)
  })
_sym_db.RegisterMessage(ModelSpec)

ModelBrief = _reflection.GeneratedProtocolMessageType('ModelBrief', (_message.Message,), {
  'DESCRIPTOR' : _MODELBRIEF,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.ModelBrief)
  })
_sym_db.RegisterMessage(ModelBrief)

ModelSpecConfiguration = _reflection.GeneratedProtocolMessageType('ModelSpecConfiguration', (_message.Message,), {
  'DESCRIPTOR' : _MODELSPECCONFIGURATION,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.ModelSpecConfiguration)
  })
_sym_db.RegisterMessage(ModelSpecConfiguration)

Model = _reflection.GeneratedProtocolMessageType('Model', (_message.Message,), {
  'DESCRIPTOR' : _MODEL,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.Model)
  })
_sym_db.RegisterMessage(Model)

ModelBuildType = _reflection.GeneratedProtocolMessageType('ModelBuildType', (_message.Message,), {
  'DESCRIPTOR' : _MODELBUILDTYPE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.ModelBuildType)
  })
_sym_db.RegisterMessage(ModelBuildType)

Source = _reflection.GeneratedProtocolMessageType('Source', (_message.Message,), {
  'DESCRIPTOR' : _SOURCE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.Source)
  })
_sym_db.RegisterMessage(Source)

ModelRegistryImage = _reflection.GeneratedProtocolMessageType('ModelRegistryImage', (_message.Message,), {
  'DESCRIPTOR' : _MODELREGISTRYIMAGE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.ModelRegistryImage)
  })
_sym_db.RegisterMessage(ModelRegistryImage)

Branch = _reflection.GeneratedProtocolMessageType('Branch', (_message.Message,), {
  'DESCRIPTOR' : _BRANCH,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.Branch)
  })
_sym_db.RegisterMessage(Branch)

GetDeploymentDetailsRequest = _reflection.GeneratedProtocolMessageType('GetDeploymentDetailsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDEPLOYMENTDETAILSREQUEST,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.GetDeploymentDetailsRequest)
  })
_sym_db.RegisterMessage(GetDeploymentDetailsRequest)

GetDeploymentDetailsResponse = _reflection.GeneratedProtocolMessageType('GetDeploymentDetailsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETDEPLOYMENTDETAILSRESPONSE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.GetDeploymentDetailsResponse)
  })
_sym_db.RegisterMessage(GetDeploymentDetailsResponse)

DeploymentDetails = _reflection.GeneratedProtocolMessageType('DeploymentDetails', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTDETAILS,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.DeploymentDetails)
  })
_sym_db.RegisterMessage(DeploymentDetails)

ListModelsMetadataRequest = _reflection.GeneratedProtocolMessageType('ListModelsMetadataRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTMODELSMETADATAREQUEST,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.ListModelsMetadataRequest)
  })
_sym_db.RegisterMessage(ListModelsMetadataRequest)

ListModelsMetadataResponse = _reflection.GeneratedProtocolMessageType('ListModelsMetadataResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTMODELSMETADATARESPONSE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.ListModelsMetadataResponse)
  })
_sym_db.RegisterMessage(ListModelsMetadataResponse)

GetModelMetadataRequest = _reflection.GeneratedProtocolMessageType('GetModelMetadataRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETMODELMETADATAREQUEST,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.GetModelMetadataRequest)
  })
_sym_db.RegisterMessage(GetModelMetadataRequest)

GetModelMetadataResponse = _reflection.GeneratedProtocolMessageType('GetModelMetadataResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETMODELMETADATARESPONSE,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.GetModelMetadataResponse)
  })
_sym_db.RegisterMessage(GetModelMetadataResponse)

EnvironmentRoutes = _reflection.GeneratedProtocolMessageType('EnvironmentRoutes', (_message.Message,), {
  'DESCRIPTOR' : _ENVIRONMENTROUTES,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.EnvironmentRoutes)
  })
_sym_db.RegisterMessage(EnvironmentRoutes)

ModelMetadata = _reflection.GeneratedProtocolMessageType('ModelMetadata', (_message.Message,), {

  'AudienceRoutesByEnvironmentEntry' : _reflection.GeneratedProtocolMessageType('AudienceRoutesByEnvironmentEntry', (_message.Message,), {
    'DESCRIPTOR' : _MODELMETADATA_AUDIENCEROUTESBYENVIRONMENTENTRY,
    '__module__' : 'qwak.models.models_pb2'
    # @@protoc_insertion_point(class_scope:qwak.models.management.ModelMetadata.AudienceRoutesByEnvironmentEntry)
    })
  ,

  'AudienceRoutesGroupedByEnvironmentEntry' : _reflection.GeneratedProtocolMessageType('AudienceRoutesGroupedByEnvironmentEntry', (_message.Message,), {
    'DESCRIPTOR' : _MODELMETADATA_AUDIENCEROUTESGROUPEDBYENVIRONMENTENTRY,
    '__module__' : 'qwak.models.models_pb2'
    # @@protoc_insertion_point(class_scope:qwak.models.management.ModelMetadata.AudienceRoutesGroupedByEnvironmentEntry)
    })
  ,
  'DESCRIPTOR' : _MODELMETADATA,
  '__module__' : 'qwak.models.models_pb2'
  # @@protoc_insertion_point(class_scope:qwak.models.management.ModelMetadata)
  })
_sym_db.RegisterMessage(ModelMetadata)
_sym_db.RegisterMessage(ModelMetadata.AudienceRoutesByEnvironmentEntry)
_sym_db.RegisterMessage(ModelMetadata.AudienceRoutesGroupedByEnvironmentEntry)

_MODELSMANAGEMENTSERVICE = DESCRIPTOR.services_by_name['ModelsManagementService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n!com.qwak.ai.management.models.apiP\001'
  _BRANCH.fields_by_name['deployed_build_id']._options = None
  _BRANCH.fields_by_name['deployed_build_id']._serialized_options = b'\030\001'
  _MODELMETADATA_AUDIENCEROUTESBYENVIRONMENTENTRY._options = None
  _MODELMETADATA_AUDIENCEROUTESBYENVIRONMENTENTRY._serialized_options = b'8\001'
  _MODELMETADATA_AUDIENCEROUTESGROUPEDBYENVIRONMENTENTRY._options = None
  _MODELMETADATA_AUDIENCEROUTESGROUPEDBYENVIRONMENTENTRY._serialized_options = b'8\001'
  _MODELMETADATA.fields_by_name['audience_routes_by_environment']._options = None
  _MODELMETADATA.fields_by_name['audience_routes_by_environment']._serialized_options = b'\030\001'
  _MODELMETADATA.fields_by_name['build']._options = None
  _MODELMETADATA.fields_by_name['build']._serialized_options = b'\030\001'
  _MODELSMANAGEMENTSERVICE.methods_by_name['CreateBranch']._options = None
  _MODELSMANAGEMENTSERVICE.methods_by_name['CreateBranch']._serialized_options = b'\210\002\001'
  _MODELSMANAGEMENTSERVICE.methods_by_name['DeleteBranch']._options = None
  _MODELSMANAGEMENTSERVICE.methods_by_name['DeleteBranch']._serialized_options = b'\210\002\001'
  _MODELSMANAGEMENTSERVICE.methods_by_name['GetDeploymentDetails']._options = None
  _MODELSMANAGEMENTSERVICE.methods_by_name['GetDeploymentDetails']._serialized_options = b'\210\002\001'
  _DEPLOYMENTMODELSTATUS._serialized_start=4336
  _DEPLOYMENTMODELSTATUS._serialized_end=4415
  _MODELSTATUS._serialized_start=4417
  _MODELSTATUS._serialized_end=4484
  _MEMORYUNIT._serialized_start=4486
  _MEMORYUNIT._serialized_end=4541
  _CREATEMODELREQUEST._serialized_start=205
  _CREATEMODELREQUEST._serialized_end=280
  _CREATEMODELRESPONSE._serialized_start=282
  _CREATEMODELRESPONSE._serialized_end=321
  _LISTMODELSREQUEST._serialized_start=323
  _LISTMODELSREQUEST._serialized_end=362
  _LISTMODELSRESPONSE._serialized_start=364
  _LISTMODELSRESPONSE._serialized_end=431
  _GETMODELREQUEST._serialized_start=433
  _GETMODELREQUEST._serialized_end=506
  _GETMODELRESPONSE._serialized_start=508
  _GETMODELRESPONSE._serialized_end=572
  _DELETEMODELREQUEST._serialized_start=574
  _DELETEMODELREQUEST._serialized_end=632
  _DELETEMODELRESPONSE._serialized_start=634
  _DELETEMODELRESPONSE._serialized_end=655
  _UPDATEMODELREQUEST._serialized_start=657
  _UPDATEMODELREQUEST._serialized_end=770
  _UPDATEMODELRESPONSE._serialized_start=772
  _UPDATEMODELRESPONSE._serialized_end=848
  _CREATEBRANCHREQUEST._serialized_start=850
  _CREATEBRANCHREQUEST._serialized_end=910
  _CREATEBRANCHRESPONSE._serialized_start=912
  _CREATEBRANCHRESPONSE._serialized_end=953
  _DELETEBRANCHREQUEST._serialized_start=955
  _DELETEBRANCHREQUEST._serialized_end=995
  _DELETEBRANCHRESPONSE._serialized_start=997
  _DELETEBRANCHRESPONSE._serialized_end=1019
  _ISMODELEXISTSREQUEST._serialized_start=1021
  _ISMODELEXISTSREQUEST._serialized_end=1099
  _ISMODELEXISTSRESPONSE._serialized_start=1101
  _ISMODELEXISTSRESPONSE._serialized_end=1140
  _MODELSPEC._serialized_start=1143
  _MODELSPEC._serialized_end=1361
  _MODELBRIEF._serialized_start=1364
  _MODELBRIEF._serialized_end=1691
  _MODELSPECCONFIGURATION._serialized_start=1693
  _MODELSPECCONFIGURATION._serialized_end=1744
  _MODEL._serialized_start=1747
  _MODEL._serialized_end=2348
  _MODELBUILDTYPE._serialized_start=2351
  _MODELBUILDTYPE._serialized_end=2501
  _SOURCE._serialized_start=2503
  _SOURCE._serialized_end=2511
  _MODELREGISTRYIMAGE._serialized_start=2513
  _MODELREGISTRYIMAGE._serialized_end=2564
  _BRANCH._serialized_start=2567
  _BRANCH._serialized_end=2802
  _BRANCH_BRANCHSTATUS._serialized_start=2734
  _BRANCH_BRANCHSTATUS._serialized_end=2802
  _GETDEPLOYMENTDETAILSREQUEST._serialized_start=2804
  _GETDEPLOYMENTDETAILSREQUEST._serialized_end=2870
  _GETDEPLOYMENTDETAILSRESPONSE._serialized_start=2872
  _GETDEPLOYMENTDETAILSRESPONSE._serialized_end=2973
  _DEPLOYMENTDETAILS._serialized_start=2976
  _DEPLOYMENTDETAILS._serialized_end=3251
  _LISTMODELSMETADATAREQUEST._serialized_start=3253
  _LISTMODELSMETADATAREQUEST._serialized_end=3300
  _LISTMODELSMETADATARESPONSE._serialized_start=3302
  _LISTMODELSMETADATARESPONSE._serialized_end=3393
  _GETMODELMETADATAREQUEST._serialized_start=3395
  _GETMODELMETADATAREQUEST._serialized_end=3438
  _GETMODELMETADATARESPONSE._serialized_start=3440
  _GETMODELMETADATARESPONSE._serialized_end=3529
  _ENVIRONMENTROUTES._serialized_start=3531
  _ENVIRONMENTROUTES._serialized_end=3635
  _MODELMETADATA._serialized_start=3638
  _MODELMETADATA._serialized_end=4334
  _MODELMETADATA_AUDIENCEROUTESBYENVIRONMENTENTRY._serialized_start=4111
  _MODELMETADATA_AUDIENCEROUTESBYENVIRONMENTENTRY._serialized_end=4216
  _MODELMETADATA_AUDIENCEROUTESGROUPEDBYENVIRONMENTENTRY._serialized_start=4218
  _MODELMETADATA_AUDIENCEROUTESGROUPEDBYENVIRONMENTENTRY._serialized_end=4334
  _MODELSMANAGEMENTSERVICE._serialized_start=4544
  _MODELSMANAGEMENTSERVICE._serialized_end=5792
# @@protoc_insertion_point(module_scope)



import uuid
from collections import defaultdict
from typing import Dict, Optional

from _qwak_proto.qwak.deployment.deployment_pb2 import (
    DeploymentBrief,
    DeploymentDetails,
    EnvironmentApplyModelTrafficResponse,
    EnvironmentDeploymentDetailsMessage,
    EnvironmentDeploymentMessage,
    EnvironmentDeploymentResultMessage,
    EnvironmentRuntimeDeploymentSettingsResultMessage,
    EnvironmentTrafficMessage,
    EnvironmentUndeploymentMessage,
    EnvironmentUndeploymentResultMessage,
    KubeDeploymentType,
    ModelDeploymentStatus,
    Variation,
)
from _qwak_proto.qwak.deployment.deployment_service_pb2 import (
    ApplyModelTrafficRequest,
    ApplyModelTrafficResponse,
    DeployModelRequest,
    DeployModelResponse,
    GetDeploymentDetailsRequest,
    GetDeploymentDetailsResponse,
    GetDeploymentHistoryRequest,
    GetDeploymentHistoryResponse,
    GetDeploymentStatusRequest,
    GetDeploymentStatusResponse,
    GetModelTrafficRequest,
    GetModelTrafficResponse,
    UndeployModelRequest,
    UndeployModelResponse,
    UpdateDeploymentRuntimeSettingsRequest,
    UpdateDeploymentRuntimeSettingsResponse,
)
from _qwak_proto.qwak.deployment.deployment_service_pb2_grpc import (
    DeploymentManagementServiceServicer,
)
from google.protobuf.timestamp_pb2 import Timestamp
from qwak_services_mock.mocks.utils.exception_handlers import raise_internal_grpc_error

FAILED_INITIATING_DEPLOYMENT_MESSAGE = "Failed initiating deployment"
DEPLOYMENT_ALREADY_RUNNING_MESSAGE = "There is already an active deployment"
FAILED_TO_UNDEPLOY_NO_ACTIVE_DEPLOYMENT = "No Active Deployment Found"
FAILED_TO_UNDEPLOY_DEPLOYMENT_IN_PROGRESS = "Deployment In Progress"
NO_ACTIVE_TRAFFIC = "No active traffic found to update"
DEFAULT_VARIATION_NAME = "default"


class DeploymentManagementServiceMock(DeploymentManagementServiceServicer):
    def __init__(self):
        self.active_deployments = defaultdict(dict)
        self.deployments_history = defaultdict(list)
        self.model_traffic = dict()
        self.model_uuid_to_active_deployment_ids = defaultdict(list)
        self.models_in_progress = set()
        self.__deployment_to_fail_on_init = set()

    def DeployModel(self, request: DeployModelRequest, context) -> DeployModelResponse:
        try:
            environments = request.environment_to_deployment.keys()
            results = dict()
            for env in environments:
                env_request_details: EnvironmentDeploymentMessage = (
                    request.environment_to_deployment[env]
                )
                if self.__should_fail_deployment(
                    env_request_details.model_id, env_request_details.build_id
                ):
                    results[env] = EnvironmentDeploymentResultMessage(
                        deployment_named_id="",
                        status=ModelDeploymentStatus.FAILED_INITIATING_DEPLOYMENT,
                        info=FAILED_INITIATING_DEPLOYMENT_MESSAGE,
                    )
                elif self.__deployment_in_progress(env_request_details.model_id, env):
                    results[env] = EnvironmentDeploymentResultMessage(
                        deployment_named_id="",
                        status=ModelDeploymentStatus.FAILED_INITIATING_DEPLOYMENT,
                        info=DEPLOYMENT_ALREADY_RUNNING_MESSAGE,
                    )
                else:
                    deployment_id = str(uuid.uuid4())
                    variation_name = self.__get_variation_name(
                        env_request_details.hosting_service
                    )
                    results[env] = EnvironmentDeploymentResultMessage(
                        deployment_named_id=deployment_id,
                        status=ModelDeploymentStatus.INITIATING_DEPLOYMENT,
                        info="",
                    )
                    self.__mark_deployment_in_progress(
                        env_request_details.model_id, env
                    )
                    deployment_key = self.get_deployment_key(
                        env_request_details.model_id, variation_name, env
                    )
                    if deployment_key in self.active_deployments:
                        prev_deployment = self.active_deployments[deployment_key]
                        prev_deployment[
                            "status"
                        ] = ModelDeploymentStatus.SUCCESSFUL_UNDEPLOYMENT
                        self.deployments_history[env_request_details.model_uuid].insert(
                            0, prev_deployment
                        )
                        self.model_uuid_to_active_deployment_ids[
                            env_request_details.model_uuid
                        ].remove(prev_deployment["deployment_id"])
                    self.active_deployments[deployment_key] = {
                        "env": env,
                        "model_id": env_request_details.model_id,
                        "build_id": env_request_details.build_id,
                        "model_uuid": env_request_details.model_uuid,
                        "deployment_id": deployment_id,
                        "hosting_service": env_request_details.hosting_service,
                        "timestamp": Timestamp(),
                        "status": ModelDeploymentStatus.INITIATING_DEPLOYMENT,
                    }
                    self.model_uuid_to_active_deployment_ids[
                        env_request_details.model_uuid
                    ].append(deployment_id)
                    self.deployments_history[env_request_details.model_uuid].insert(
                        0, self.active_deployments[deployment_key]
                    )
                    if (
                        env_request_details.hosting_service.kube_deployment.kube_deployment_type
                        == KubeDeploymentType.ONLINE
                    ):
                        self.model_traffic[
                            (env_request_details.model_id, env)
                        ] = (
                            env_request_details.hosting_service.kube_deployment.serving_strategy.realtime_config.traffic_config
                        )
                    else:
                        try:
                            del self.model_traffic[(env_request_details.model_id, env)]
                        except Exception:
                            pass

            return DeployModelResponse(environment_to_deployment_result=results)
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def UndeployModel(
        self, request: UndeployModelRequest, context
    ) -> UndeployModelResponse:
        try:
            environments = request.environment_to_undeployment.keys()
            results = dict()
            for env in environments:
                details: EnvironmentUndeploymentMessage = (
                    request.environment_to_undeployment[env]
                )
                deployment_key = self.get_deployment_key(
                    details.model_id, details.variation_name, env
                )
                if deployment_key not in self.active_deployments:
                    results[env] = EnvironmentUndeploymentResultMessage(
                        status=ModelDeploymentStatus.FAILED_UNDEPLOYMENT,
                        info=FAILED_TO_UNDEPLOY_NO_ACTIVE_DEPLOYMENT,
                    )
                elif self.__deployment_in_progress(details.model_id, env):
                    results[env] = EnvironmentUndeploymentResultMessage(
                        status=ModelDeploymentStatus.FAILED_UNDEPLOYMENT,
                        info=FAILED_TO_UNDEPLOY_DEPLOYMENT_IN_PROGRESS,
                    )
                else:
                    deployment_details = self.active_deployments[deployment_key]
                    deployment_details[
                        "status"
                    ] = ModelDeploymentStatus.SUCCESSFUL_UNDEPLOYMENT
                    self.deployments_history[details.model_uuid].insert(
                        0, deployment_details
                    )
                    self.model_uuid_to_active_deployment_ids[details.model_uuid].remove(
                        deployment_details["deployment_id"]
                    )
                    del self.active_deployments[deployment_key]
                    if details.traffic_config:
                        self.model_traffic[
                            (details.model_id, env)
                        ] = details.traffic_config
                    else:
                        try:
                            del self.model_traffic[(details.model_id, env)]
                        except Exception:
                            pass
                    results[env] = EnvironmentUndeploymentResultMessage(
                        status=ModelDeploymentStatus.SUCCESSFUL_UNDEPLOYMENT, info=""
                    )

            return UndeployModelResponse(environment_to_undeployment_result=results)
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def GetDeploymentStatus(
        self, request: GetDeploymentStatusRequest, context
    ) -> GetDeploymentStatusResponse:
        try:
            deployment = self.get_deployment_by_id(request.deployment_named_id)
            status = (
                deployment["status"]
                if deployment
                else ModelDeploymentStatus.SUCCESSFUL_UNDEPLOYMENT
            )
            return GetDeploymentStatusResponse(status=status)
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def GetDeploymentDetails(
        self, request: GetDeploymentDetailsRequest, context
    ) -> GetDeploymentDetailsResponse:
        try:
            result = {}
            environment_deployment_details_results = defaultdict(list)
            active_deployment_ids = self.model_uuid_to_active_deployment_ids[
                request.model_uuid
            ]
            for active_deployment in active_deployment_ids:
                deployment_details = self.get_deployment_by_id(active_deployment)
                deployment_details_proto: DeploymentDetails = (
                    self.__deployment_details_to_proto(deployment_details)
                )
                environment_deployment_details_results[
                    deployment_details_proto.environment_id
                ].append(deployment_details_proto)

            for env, deployments_list in environment_deployment_details_results.items():
                result[env] = EnvironmentDeploymentDetailsMessage(
                    deployments_details=deployments_list,
                    fallback_variation=DEFAULT_VARIATION_NAME,
                )

            return GetDeploymentDetailsResponse(
                environment_to_deployment_details=result
            )
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def GetDeploymentHistory(
        self, request: GetDeploymentHistoryRequest, context
    ) -> GetDeploymentHistoryResponse:
        try:
            result = []
            for deployment_details in self.deployments_history[request.model_uuid]:
                deployment_brief_proto: DeploymentBrief = (
                    self.__deployment_brief_to_proto(deployment_details)
                )
                result.append(deployment_brief_proto)
            return GetDeploymentHistoryResponse(deployments=result)
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def UpdateDeploymentRuntimeSettings(
        self, request: UpdateDeploymentRuntimeSettingsRequest, context
    ) -> UpdateDeploymentRuntimeSettingsResponse:
        try:
            results = {}
            for deployment_id in self.model_uuid_to_active_deployment_ids[
                request.model_uuid
            ]:
                deployment_details = self.get_deployment_by_id(deployment_id)
                if request.variation_name:
                    if (
                        self.__get_variation_name(deployment_details["hosting_service"])
                        == request.variation_name
                    ):
                        results[
                            deployment_details["env"]
                        ] = EnvironmentRuntimeDeploymentSettingsResultMessage()
                else:
                    results[
                        deployment_details["env"]
                    ] = EnvironmentRuntimeDeploymentSettingsResultMessage()
            return UpdateDeploymentRuntimeSettingsResponse(
                environment_to_runtime_deployment_settings_response=results
            )
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def ApplyModelTraffic(
        self, request: ApplyModelTrafficRequest, context
    ) -> ApplyModelTrafficResponse:
        try:
            result = {}
            for env in request.environment_ids:
                if (request.model_id, env) in self.model_traffic:
                    del self.model_traffic[(request.model_id, env)].variations[:]
                    self.model_traffic[(request.model_id, env)].variations.extend(
                        request.variations
                    )
                    result[env] = EnvironmentApplyModelTrafficResponse()
                else:
                    result[env] = EnvironmentApplyModelTrafficResponse(
                        info=NO_ACTIVE_TRAFFIC
                    )
            return ApplyModelTrafficResponse(
                environment_to_apply_model_traffic_response=result
            )
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def GetModelTraffic(
        self, request: GetModelTrafficRequest, context
    ) -> GetModelTrafficResponse:
        try:
            result = {}
            for model_and_env, traffic in self.model_traffic.items():
                model_id, env = model_and_env
                if model_id != request.model_id:
                    continue

                result[env] = EnvironmentTrafficMessage(variations=traffic.variations)

            return GetModelTrafficResponse(environment_to_model_traffic=result)
        except Exception as e:
            raise_internal_grpc_error(context, e)

    @staticmethod
    def get_deployment_key(model_id: str, variation_name: str, env: str):
        return model_id, variation_name, env

    @staticmethod
    def __get_variation_name(hosting_service):
        if (
            hosting_service.kube_deployment
            and hosting_service.kube_deployment.serving_strategy
            and hosting_service.kube_deployment.serving_strategy.realtime_config
        ):
            variation_name = (
                hosting_service.kube_deployment.serving_strategy.realtime_config.traffic_config.selected_variation_name
            )
        else:
            variation_name = DEFAULT_VARIATION_NAME
        return variation_name

    def set_deployment_to_fail_on_init(self, model_id: str, build_id: str):
        self.__deployment_to_fail_on_init.add((model_id, build_id))

    def __should_fail_deployment(self, model_id: str, build_id: str) -> bool:
        return (model_id, build_id) in self.__deployment_to_fail_on_init

    def __deployment_in_progress(self, model_id: str, environment: str) -> bool:
        return (model_id, environment) in self.models_in_progress

    def __mark_deployment_in_progress(self, model_id, environment: str):
        self.models_in_progress.add((model_id, environment))

    def get_deployment_by_id(self, deployment_id: str) -> Optional[Dict]:
        for deployment_details in self.active_deployments.values():
            if deployment_details["deployment_id"] == deployment_id:
                return deployment_details

        for deployments_history_list in self.deployments_history.values():
            for deployment_details in deployments_history_list:
                if deployment_details["deployment_id"] == deployment_id:
                    return deployment_details

        return None

    def mark_deployment_successful(
        self,
        model_id: str,
        env: str,
        variation: str = DEFAULT_VARIATION_NAME,
    ):
        deployment_key = self.get_deployment_key(model_id, variation, env)
        if deployment_key in self.active_deployments:
            old_status = self.active_deployments[(model_id, variation, env)]["status"]
            if old_status != ModelDeploymentStatus.INITIATING_DEPLOYMENT:
                raise Exception(
                    f"Cannot modify deployment status from {old_status} to SUCCESSFUL"
                )
            self.models_in_progress.remove((model_id, env))
            self.active_deployments[deployment_key][
                "status"
            ] = ModelDeploymentStatus.SUCCESSFUL_DEPLOYMENT
            return True
        return False

    def __deployment_details_to_proto(self, deployment_details):
        return DeploymentDetails(
            build_id=deployment_details["build_id"],
            number_of_pods=deployment_details[
                "hosting_service"
            ].kube_deployment.deployment_size.number_of_pods,
            cpu_fraction=deployment_details[
                "hosting_service"
            ].kube_deployment.deployment_size.cpu,
            memory_amount=deployment_details[
                "hosting_service"
            ].kube_deployment.deployment_size.memory_amount,
            memory_units=deployment_details[
                "hosting_service"
            ].kube_deployment.deployment_size.memory_units,
            number_of_workers=deployment_details[
                "hosting_service"
            ].kube_deployment.advanced_deployment_options.number_of_http_server_workers,
            http_request_timeout_ms=deployment_details[
                "hosting_service"
            ].kube_deployment.advanced_deployment_options.http_request_timeout_ms,
            kube_deployment_type=deployment_details[
                "hosting_service"
            ].kube_deployment.kube_deployment_type,
            serving_strategy=deployment_details[
                "hosting_service"
            ].kube_deployment.serving_strategy,
            variation=Variation(
                name=self.__get_variation_name(deployment_details["hosting_service"])
            ),
            custom_iam_role_arn=deployment_details[
                "hosting_service"
            ].kube_deployment.advanced_deployment_options.custom_iam_role_arn,
            max_batch_size=deployment_details[
                "hosting_service"
            ].kube_deployment.advanced_deployment_options.max_batch_size,
            gpu_type=deployment_details[
                "hosting_service"
            ].kube_deployment.deployment_size.gpu_resources.gpu_type,
            gpu_amount=deployment_details[
                "hosting_service"
            ].kube_deployment.deployment_size.gpu_resources.gpu_amount,
            environment_id=deployment_details["env"],
            available_replicas=2,
            deployment_id=deployment_details["deployment_id"],
        )

    def __deployment_brief_to_proto(self, deployment_details):
        return DeploymentBrief(
            build_id=deployment_details["build_id"],
            status=deployment_details["status"],
            deployment_timestamp=deployment_details["timestamp"],
            environment_id=deployment_details["env"],
            variation_name=deployment_details[
                "hosting_service"
            ].kube_deployment.serving_strategy.realtime_config.traffic_config.selected_variation_name,
            variation_protect_state=deployment_details[
                "hosting_service"
            ].kube_deployment.serving_strategy.realtime_config.traffic_config.selected_variation_protect_state,
            deployment_id=deployment_details["deployment_id"],
        )

    def given_deployment(
        self,
        model_id: str,
        environment_id: str,
        status: int,
        model_uuid: str,
        deployment_id: str,
    ) -> str:
        deployment_details = self.active_deployments.get((model_id, "", environment_id))
        deployment_details["status"] = status
        deployment_details["model_uuid"] = [model_uuid]
        self.active_deployments[(model_id, "", environment_id)] = deployment_details
        self.model_uuid_to_active_deployment_ids = defaultdict(list)
        self.model_uuid_to_active_deployment_ids[model_uuid] = [deployment_id]
        return deployment_id



import collections
import functools
import inspect
from dataclasses import dataclass, field
from datetime import datetime
from typing import TYPE_CHECKING, List, Optional, Tuple, Union

from _qwak_proto.qwak.feature_store.features.execution_pb2 import (
    StreamingExecutionSpec as ProtoStreamingExecutionSpec,
)
from _qwak_proto.qwak.feature_store.features.feature_set_pb2 import (
    FeatureSetSpec as ProtoFeatureSetSpec,
)
from _qwak_proto.qwak.feature_store.features.feature_set_types_pb2 import (
    AggregationSpec as ProtoAggregationSpec,
    FeatureSetType as ProtoFeatureSetType,
    StreamingAggregationFeatureSet as ProtoStreamingAggregationFeatureSet,
    StreamingFeatureSetV1 as ProtoStreamingFeatureSetV1,
)
from _qwak_proto.qwak.feature_store.sources.streaming_pb2 import (
    StreamingSource,
    StreamingSource as ProtoStreamingSource,
)
from qwak.clients.feature_store import FeatureRegistryClient
from qwak.exceptions import QwakException
from qwak.feature_store._common.artifact_utils import ArtifactSpec, ArtifactsUploader
from qwak.feature_store.entities.entity import Entity
from qwak.feature_store.feature_sets._utils._featureset_utils import FeaturesetUtils
from qwak.feature_store.feature_sets.base_feature_set import BaseFeatureSet
from qwak.feature_store.feature_sets.execution_spec import ClusterTemplate
from qwak.feature_store.feature_sets.metadata import (
    Metadata,
    get_metadata_from_function,
    set_metadata_on_function,
)
from qwak.feature_store.feature_sets.streaming_backfill import (
    BackfillBatchDataSourceSpec,
    StreamingBackfill,
)
from qwak.feature_store.feature_sets.transformations import (
    BaseTransformation,
    PySparkTransformation,
    SparkSqlTransformation,
    Window,
)
from qwak.feature_store.sinks.base import BaseSink
from qwak.feature_store.sinks.streaming.factory import StreamingSinkFactory
from qwak.feature_store.validations.validation_options import (
    FeatureSetValidationOptions,
)
from typeguard import typechecked

if TYPE_CHECKING:
    try:
        import pandas as pd
    except ImportError:
        pass

_OFFLINE_SCHEDULING_ATTRIBUTE = "_qwak_offline_scheduling"
_OFFLINE_CLUSTER_SPEC = "_qwak_offline_cluster_specification"
_ONLINE_TRIGGER_INTERVAL = "_qwak_online_trigger_interval"
_ONLINE_CLUSTER_SPEC = "_qwak_online_cluster_specification"
_METADATA_ = "_qwak_online_cluster_specification"


@typechecked
def feature_set(
    *,
    data_sources: List[str],
    timestamp_column_name: str,
    offline_scheduling_policy: Optional[str] = "*/30 * * * *",
    online_trigger_interval: Optional[int] = 5,
    name: Optional[str] = None,
    entity: Optional[str] = None,
    key: Optional[str] = None,
    auxiliary_sinks: List[BaseSink] = [],
):
    """
    Creates a streaming feature set for the specified entity using the given streaming data sources.

    A streaming feature set allows for real-time updates of features from live data sources, letting ML models access
    the most recent values without waiting for batch updates.

    :param entity: The name of the entity for which the feature set is being created. An entity typically represents a
                   unique object or concept, like 'user', 'product', etc. Entity and key are mutually exclusive.
    :param key: a column name in the feature set which is the key. Entity and key are mutually exclusive.
    :param data_sources: A list of references to the data sources from which the feature values will be streamed.
                                Each data source should be capable of providing data in a streaming manner.
    :param timestamp_column_name: The name of the column in the data source that contains timestamp information. This
                                  is used to order the data chronologically and ensure that the feature values are
                                  updated in the correct order.
    :param offline_scheduling_policy: Defines the offline ingestion policy - which affects the data freshness of
                                      the offline store. defaults to */30 * * * * (every 30 minutes)
    :param online_trigger_interval: Defines the online ingestion policy  - which affects the data freshness of
                                      the online store. defaults to 5 seconds
    :param name: An optional name for the feature set. If not provided, the name of the function will be used.
    :param  auxiliary_sinks: list of auxiliary sinks. Not supported with Aggregations
    Example:

    ... code-block:: python

        @streaming.feature_set(
            entity="users",
            data_sources=["users_registration_stream"],
            timestamp_column_name="reg_date"
        )
        def user_streaming_features():
            return SparkSqlTransformation("SELECT user_id, reg_country, reg_date FROM data_source")
    """

    def decorator(function):
        user_transformation = function()
        FeaturesetUtils.validate_base_featureset_decorator(
            user_transformation=user_transformation, entity=entity, key=key
        )

        FeaturesetUtils.validate_streaming_featureset_decorator(
            online_trigger_interval=online_trigger_interval,
            offline_scheduling_policy=offline_scheduling_policy,
        )

        streaming_backfill: Optional[
            StreamingBackfill
        ] = StreamingBackfill.get_streaming_backfill_from_function(function=function)

        fs_name = name or function.__name__
        streaming_feature_set = StreamingFeatureSet(
            name=fs_name,
            entity=entity if entity else None,
            key=key if key else None,
            data_sources=data_sources,
            timestamp_column_name=timestamp_column_name,
            transformation=user_transformation,
            metadata=get_metadata_from_function(
                function, description=fs_name, display_name=fs_name
            ),
            online_trigger_interval=(
                online_trigger_interval if online_trigger_interval else 5
            ),
            offline_scheduling_policy=(
                offline_scheduling_policy
                if offline_scheduling_policy
                else "*/30 * * * *"
            ),
            offline_cluster_template=getattr(
                function, _OFFLINE_CLUSTER_SPEC, ClusterTemplate.SMALL
            ),
            online_cluster_template=getattr(
                function, _ONLINE_CLUSTER_SPEC, ClusterTemplate.SMALL
            ),
            backfill=streaming_backfill,
            __instance_module_path__=inspect.stack()[1].filename,
            auxiliary_sinks=auxiliary_sinks,
        )

        functools.update_wrapper(streaming_feature_set, user_transformation)
        return streaming_feature_set

    return decorator


@typechecked
def execution_specification(
    *,
    online_cluster_template: Optional[ClusterTemplate] = None,
    offline_cluster_template: Optional[ClusterTemplate] = None,
):
    """
    Set the execution specification of the cluster running the feature set

    :param online_cluster_template: Predefined template sizes
    :param offline_cluster_template: Predefined template sizes

    Cluster template example:

    ... code-block:: python
        @streaming.feature_set(entity="users", data_sources=["streaming_users_source"])
        @streaming.execution_specification(
                offline_cluster_template=ClusterTemplate.MEDIUM,
                online_cluster_template=ClusterTemplate.MEDIUM)
        def user_streaming_features():
            return SparkSqlTransformation("SELECT user_id, age, timestamp FROM streaming_users_source"
    """

    def decorator(user_transformation):
        setattr(user_transformation, _ONLINE_CLUSTER_SPEC, online_cluster_template)

        setattr(user_transformation, _OFFLINE_CLUSTER_SPEC, offline_cluster_template)

        return user_transformation

    return decorator


@typechecked
def backfill(
    *,
    start_date: datetime,
    end_date: datetime,
    data_sources: Union[List[str], List[BackfillBatchDataSourceSpec]],
    backfill_transformation: SparkSqlTransformation,
    backfill_cluster_template: Optional[ClusterTemplate] = ClusterTemplate.SMALL,
):
    """
    Configures optional backfill specification for a streaming featureset. Currently available for streaming
    aggregation featuresets only.

    :param start_date: backfill start date
    :param end_date: backfill end date
    :param data_sources: a list of dict representations connecting each batch source name to its requested time range
    :param backfill_transformation: a backfill SQL transformation. Only SQL transformations are supported for backfill
    :param backfill_cluster_template: an optional cluster specification for the backfill job. Defaults to SMALL.

    Example:

    ... code-block:: python

        @streaming.feature_set(
            entity="users",
            data_sources=["users_registration_stream"],
            timestamp_column_name="reg_date"
        )
        @streaming.backfill(
            start_date=start_date=(2022,01,01,0,0,0),
            end_date=start_date=(2023,09,01,0,0,0),
            data_sources=[{"batch_backfill_source_name": BackfillSourceRange(start_date=(2023,01,01,0,0,0),
                                                                             end_date=(2023,08,01,0,0,0))}],
            backfill_cluster_template=ClusterTemplate.SMALL
            backfill_transformation=SparkSqlTransformation("SELECT user_id, reg_country, reg_date FROM backfill_data_source")
        )
        def user_streaming_features():
            return SparkSqlTransformation("SELECT user_id, reg_country, reg_date FROM data_source")
    """

    def decorator(function):
        _validate_decorator_ordering(function)
        StreamingBackfill.set_streaming_backfill_on_function(
            function=function,
            start_date=start_date,
            end_date=end_date,
            data_sources=data_sources,
            backfill_transformation=backfill_transformation,
            backfill_cluster_template=backfill_cluster_template,
        )

        return function

    return decorator


@typechecked
def metadata(
    *,
    owner: Optional[str] = None,
    description: Optional[str] = None,
    display_name: Optional[str] = None,
    version_comment: Optional[str] = None,
):
    """
    Sets additional user provided metadata

    :param owner: feature set owner
    :param description: General description of the feature set
    :param display_name: Human readable name of the feature set
    :param version_comment: Comment which describes the version

    Example:

    ... code-block:: python

        @streaming.feature_set(
            entity="users",
            data_sources=["users_registration_stream"],
            timestamp_column_name="reg_date"
        )
        @streaming.metadata(
            owner="datainfra@qwak.com",
            display_name="User Streaming Features",
            description="Users feature from the Kafka topic of users registration stream",
        )
        def user_streaming_features():
            return SparkSqlTransformation("SELECT user_id, reg_country, reg_date FROM data_source")

    """

    def decorator(function):
        _validate_decorator_ordering(function)
        set_metadata_on_function(
            function=function,
            owner=owner,
            description=description,
            display_name=display_name,
            version_comment=version_comment,
        )

        return function

    return decorator


def _validate_decorator_ordering(function):
    if isinstance(function, StreamingFeatureSet):
        raise ValueError(
            "Wrong decorator ordering - @streaming.feature_set should be the top most decorator"
        )


@dataclass
class StreamingFeatureSet(BaseFeatureSet):
    timestamp_column_name: str = str()
    online_trigger_interval: int = int()
    offline_scheduling_policy: str = str()
    transformation: Optional[BaseTransformation] = None
    offline_cluster_template: Optional[ClusterTemplate] = None
    online_cluster_template: Optional[ClusterTemplate] = None
    metadata: Optional[Metadata] = None
    backfill: Optional[StreamingBackfill] = None
    auxiliary_sinks: List[BaseSink] = field(default_factory=lambda: [])

    def __post_init__(self):
        self._validate()

    @classmethod
    def _from_proto(cls, proto: ProtoFeatureSetSpec):
        streaming_def: ProtoStreamingFeatureSetV1 = (
            proto.feature_set_type.streaming_feature_set_v1
        )

        return cls(
            name=proto.name,
            entity=Entity._from_proto(proto.entity).name,
            data_sources=[ds.name for ds in streaming_def.data_sources],
            timestamp_column_name=streaming_def.timestamp_column_name,
            online_trigger_interval=streaming_def.online_trigger_interval,
            offline_scheduling_policy=streaming_def.offline_scheduling_policy,
            transformation=BaseTransformation._from_proto(streaming_def.transformation),
            offline_cluster_template=ClusterTemplate.from_cluster_template_number(
                streaming_def.execution_spec.offline_cluster_template
            ),
            online_cluster_template=ClusterTemplate.from_cluster_template_number(
                streaming_def.execution_spec.online_cluster_template
            ),
            metadata=Metadata.from_proto(proto.metadata),
            auxiliary_sinks=[
                StreamingSinkFactory.get_streaming_sink(proto)
                for proto in streaming_def.auxiliary_sinks
            ],
        )

    def _get_data_sources(
        self, feature_registry: FeatureRegistryClient
    ) -> List[ProtoStreamingSource]:
        sources: List[ProtoStreamingSource] = list()

        for name in self.data_sources:
            ds = feature_registry.get_data_source_by_name(name)
            if not ds:
                raise QwakException(
                    f"Trying to register a featureset with a non existing data source {name}"
                )
            else:
                sources.append(
                    ds.data_source.data_source_definition.data_source_spec.stream_source
                )
        return sources

    def _to_proto(
        self,
        git_commit,
        features,
        feature_registry: FeatureRegistryClient,
        artifact_url: Optional[str] = None,
        **kwargs,
    ) -> Tuple[ProtoFeatureSetSpec, Optional[str]]:
        maybe_initial_tile_size: Optional[int] = self._validate_streaming_aggregation()

        data_sources: List[StreamingSource] = self._get_data_sources(feature_registry)

        if not artifact_url:
            artifact: Optional[ArtifactSpec] = ArtifactsUploader.get_artifact_spec(
                transformation=self.transformation,
                featureset_name=self.name,
                __instance_module_path__=self.__instance_module_path__,
            )
            if artifact:
                artifact_url = ArtifactsUploader.upload(artifact)

        proto_featureset_type: ProtoFeatureSetType
        if maybe_initial_tile_size is None:
            # row-level streaming
            proto_featureset_type = self._get_streaming_featureset_proto(
                artifact_url=artifact_url, streaming_sources=data_sources
            )
        else:
            # streaming aggregation
            proto_featureset_type = self._get_streaming_aggregation_featureset_proto(
                artifact_url=artifact_url,
                streaming_sources=data_sources,
                feature_registry=feature_registry,
                initial_tile_size=maybe_initial_tile_size,
            )

        return (
            ProtoFeatureSetSpec(
                name=self.name,
                metadata=self.metadata.to_proto(),
                git_commit=git_commit,
                features=features,
                entity=self._get_entity_definition(feature_registry),
                feature_set_type=proto_featureset_type,
            ),
            artifact_url,
        )

    def _validate(self):
        import croniter

        super()._validate()

        # verify the cron expression is valid
        if not self.offline_scheduling_policy or not croniter.croniter.is_valid(
            self.offline_scheduling_policy
        ):
            raise QwakException(
                f"offline scheduling policy "
                f"'{self.offline_scheduling_policy}'"
                f" is not a valid cron expression"
            )

        # verify the online scheduling policy is valid
        if self.online_trigger_interval < 0:
            raise QwakException(
                f"Value '{self.online_trigger_interval}'"
                f" is not a legal online scheduling policy, "
                f"only non-negative integers are allowed"
            )

        # verify timestamp_col_name was set
        if not self.timestamp_column_name:
            raise QwakException("'timestamp_col_name' field must be set")

        is_streaming_agg = bool(self._validate_streaming_aggregation())

        # if sinks were configured, make sure it's not streaming-agg
        if len(self.auxiliary_sinks) > 0 and is_streaming_agg:
            raise QwakException(
                "Auxiliary Sinks Are not supported in Streaming Aggregation Feature Sets"
            )

        # streaming backfill not yet supported for row-level streaming
        if self.backfill and not is_streaming_agg:
            raise QwakException(
                "Streaming backfill is only supported for streaming aggregation feature sets at the moment"
            )

        # Validate transformation is PySpark when multiple data sources are used
        if len(self.data_sources) > 1 and not isinstance(
            self.transformation, PySparkTransformation
        ):
            raise QwakException(
                "When using multiple data sources, only `PySparkTransformation` is allowed."
            )

    def _validate_streaming_aggregation(self) -> Optional[int]:
        if not (self.transformation.windows or self.transformation.aggregations):
            # definitely not streaming aggregation
            return None

        # at least 1 window and/or at least 1 aggregate - so we need to verify
        # it's a valid streaming aggregation definition
        if not self.transformation.windows:
            raise QwakException(
                "When specifying aggregations, at least one time window must be defined"
            )

        if not self.transformation.aggregations:
            raise QwakException(
                "When specifying time windows, at least one aggregation must be defined"
            )

        (
            initial_tile_size,
            min_window,
        ) = StreamingFeatureSet._get_default_slide_period(self.transformation.windows)

        if initial_tile_size > max(10.0, (min_window / 10)):
            raise QwakException(
                "Windows with such different cardinality (For example years vs days) can't be in "
                "the same feature set. Please separate to different feature sets."
            )

        # Validate no feature duplications exist

        # Get feature names, with or without aliases
        final_feature_names = self.transformation.get_features_names()

        # Look for duplicated feature names
        group_by_feature_name_to_count = collections.Counter(final_feature_names)
        features_with_duplicates = [
            feature
            for feature, count in group_by_feature_name_to_count.items()
            if count > 1
        ]
        if features_with_duplicates:
            error_message_str = ""
            for dup in features_with_duplicates:
                error_message_str += (
                    f"{repr(dup)} feature, appears more than once in Aggregations.\n"
                )
            raise QwakException(error_message_str)

        # Validate the backfill, if defined
        if self.backfill:
            self.backfill._validate_tile_size(initial_tile_size)

        return initial_tile_size

    def _validate_streaming_aggregation_backfill(self):
        initial_tile_size, _ = StreamingFeatureSet._get_default_slide_period(
            self.transformation.windows
        )

        self.backfill._validate_tile_size(initial_tile_size)

    @staticmethod
    def _get_default_slide_period(
        windows: List[Window], minimum_time_window_seconds: int = 10
    ):
        """
        Tile size is the max between the `minimum_time_window_seconds`,
        and the round down of the max window size divided by 2048.
        By dividing by 2048 we promise that the max aggregation time contains
        up to 2048 different tiles, which limit the amount of keys that exists in the Online Store.
        :param windows:
        :param minimum_time_window_seconds:
        :return: tuple of tile size and the minimum window size seconds
        """
        time_windows_in_seconds = [w.length * w.seconds_in_time_unit for w in windows]
        max_window = max(time_windows_in_seconds)
        min_window = min(time_windows_in_seconds)

        if min_window < minimum_time_window_seconds:
            raise QwakException(
                f"The minimum time window is {minimum_time_window_seconds} seconds"
            )

        def round(k):
            return k - (k % minimum_time_window_seconds)

        initial_tile_size = round(max((max_window / 2048), minimum_time_window_seconds))

        return int(initial_tile_size), min_window

    def get_sample(
        self,
        number_of_rows: int = 10,
        validation_options: Optional[FeatureSetValidationOptions] = None,
    ) -> "pd.DataFrame":
        return super().get_sample(
            number_of_rows=number_of_rows, validation_options=validation_options
        )

    def _get_streaming_featureset_proto(
        self, artifact_url: Optional[str], streaming_sources: List[StreamingSource]
    ) -> ProtoFeatureSetType:
        return ProtoFeatureSetType(
            streaming_feature_set_v1=ProtoStreamingFeatureSetV1(
                transformation=self.transformation._to_proto(
                    artifact_path=artifact_url
                ),
                data_sources=streaming_sources,
                execution_spec=ProtoStreamingExecutionSpec(
                    online_cluster_template=ClusterTemplate.to_proto(
                        self.online_cluster_template
                    ),
                    offline_cluster_template=ClusterTemplate.to_proto(
                        self.offline_cluster_template
                    ),
                ),
                timestamp_column_name=self.timestamp_column_name,
                online_trigger_interval=self.online_trigger_interval,
                offline_scheduling_policy=self.offline_scheduling_policy,
                auxiliary_sinks=[
                    s.to_proto_streaming_sink() for s in self.auxiliary_sinks
                ],
            )
        )

    def _get_streaming_aggregation_featureset_proto(
        self,
        artifact_url: Optional[str],
        streaming_sources: List[StreamingSource],
        feature_registry: FeatureRegistryClient,
        initial_tile_size: int,
    ) -> ProtoFeatureSetType:
        return ProtoFeatureSetType(
            streaming_aggregation_feature_set=ProtoStreamingAggregationFeatureSet(
                transformation=self.transformation._to_proto(
                    artifact_path=artifact_url
                ),
                data_sources=streaming_sources,
                execution_spec=ProtoStreamingExecutionSpec(
                    online_cluster_template=ClusterTemplate.to_proto(
                        self.online_cluster_template
                    ),
                    offline_cluster_template=ClusterTemplate.to_proto(
                        self.offline_cluster_template
                    ),
                ),
                timestamp_column_name=self.timestamp_column_name,
                online_trigger_interval=self.online_trigger_interval,
                compaction_scheduling_policy=self.offline_scheduling_policy,
                aggregation_spec=ProtoAggregationSpec(
                    slide_seconds=initial_tile_size,
                    allowed_late_arrival_seconds=60 * 10,
                    aggregations=self.transformation._get_aggregations_proto(),
                ),
                backfill_spec=(
                    self.backfill._to_proto(
                        feature_registry=feature_registry,
                        original_instance_module_path=self.__instance_module_path__,
                        featureset_name=self.name,
                    )
                    if self.backfill
                    else None
                ),
            )
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/monitoring/v0/alerting_channel_management_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.monitoring.v0 import alerting_channel_pb2 as qwak_dot_monitoring_dot_v0_dot_alerting__channel__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<qwak/monitoring/v0/alerting_channel_management_service.proto\x12\x12qwak.monitoring.v0\x1a)qwak/monitoring/v0/alerting_channel.proto\"a\n\x1c\x43reateAlertingChannelRequest\x12\x41\n\x07options\x18\x01 \x01(\x0b\x32\x30.qwak.monitoring.v0.AlertingChannelCreateOptions\"\x1f\n\x1d\x43reateAlertingChannelResponse\"a\n\x1cUpdateAlertingChannelRequest\x12\x41\n\x07options\x18\x01 \x01(\x0b\x32\x30.qwak.monitoring.v0.AlertingChannelUpdateOptions\"\x1f\n\x1dUpdateAlertingChannelResponse\"+\n\x1dGetAlertingChannelRequestById\x12\n\n\x02id\x18\x01 \x01(\t\"e\n\x1eGetAlertingChannelResponseById\x12\x43\n\x0b\x64\x65scription\x18\x01 \x01(\x0b\x32..qwak.monitoring.v0.AlertingChannelDescription\"/\n\x1fGetAlertingChannelRequestByName\x12\x0c\n\x04name\x18\x01 \x01(\t\"g\n GetAlertingChannelResponseByName\x12\x43\n\x0b\x64\x65scription\x18\x01 \x01(\x0b\x32..qwak.monitoring.v0.AlertingChannelDescription\"\x1c\n\x1aListAlertingChannelRequest\"b\n\x1bListAlertingChannelResponse\x12\x43\n\x0b\x64\x65scription\x18\x01 \x03(\x0b\x32..qwak.monitoring.v0.AlertingChannelDescription\",\n\x1c\x44\x65leteAlertingChannelRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x1f\n\x1d\x44\x65leteAlertingChannelResponse2\x9d\x06\n AlertingChannelManagementService\x12|\n\x15\x43reateAlertingChannel\x12\x30.qwak.monitoring.v0.CreateAlertingChannelRequest\x1a\x31.qwak.monitoring.v0.CreateAlertingChannelResponse\x12|\n\x15UpdateAlertingChannel\x12\x30.qwak.monitoring.v0.UpdateAlertingChannelRequest\x1a\x31.qwak.monitoring.v0.UpdateAlertingChannelResponse\x12\x7f\n\x16GetAlertingChannelById\x12\x31.qwak.monitoring.v0.GetAlertingChannelRequestById\x1a\x32.qwak.monitoring.v0.GetAlertingChannelResponseById\x12\x85\x01\n\x18GetAlertingChannelByName\x12\x33.qwak.monitoring.v0.GetAlertingChannelRequestByName\x1a\x34.qwak.monitoring.v0.GetAlertingChannelResponseByName\x12v\n\x13ListAlertingChannel\x12..qwak.monitoring.v0.ListAlertingChannelRequest\x1a/.qwak.monitoring.v0.ListAlertingChannelResponse\x12|\n\x15\x44\x65leteAlertingChannel\x12\x30.qwak.monitoring.v0.DeleteAlertingChannelRequest\x1a\x31.qwak.monitoring.v0.DeleteAlertingChannelResponseB2\n\x1d\x63om.qwak.ai.monitoring.api.v0P\x01Z\x0f.;monitoring_v0b\x06proto3')



_CREATEALERTINGCHANNELREQUEST = DESCRIPTOR.message_types_by_name['CreateAlertingChannelRequest']
_CREATEALERTINGCHANNELRESPONSE = DESCRIPTOR.message_types_by_name['CreateAlertingChannelResponse']
_UPDATEALERTINGCHANNELREQUEST = DESCRIPTOR.message_types_by_name['UpdateAlertingChannelRequest']
_UPDATEALERTINGCHANNELRESPONSE = DESCRIPTOR.message_types_by_name['UpdateAlertingChannelResponse']
_GETALERTINGCHANNELREQUESTBYID = DESCRIPTOR.message_types_by_name['GetAlertingChannelRequestById']
_GETALERTINGCHANNELRESPONSEBYID = DESCRIPTOR.message_types_by_name['GetAlertingChannelResponseById']
_GETALERTINGCHANNELREQUESTBYNAME = DESCRIPTOR.message_types_by_name['GetAlertingChannelRequestByName']
_GETALERTINGCHANNELRESPONSEBYNAME = DESCRIPTOR.message_types_by_name['GetAlertingChannelResponseByName']
_LISTALERTINGCHANNELREQUEST = DESCRIPTOR.message_types_by_name['ListAlertingChannelRequest']
_LISTALERTINGCHANNELRESPONSE = DESCRIPTOR.message_types_by_name['ListAlertingChannelResponse']
_DELETEALERTINGCHANNELREQUEST = DESCRIPTOR.message_types_by_name['DeleteAlertingChannelRequest']
_DELETEALERTINGCHANNELRESPONSE = DESCRIPTOR.message_types_by_name['DeleteAlertingChannelResponse']
CreateAlertingChannelRequest = _reflection.GeneratedProtocolMessageType('CreateAlertingChannelRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEALERTINGCHANNELREQUEST,
  '__module__' : 'qwak.monitoring.v0.alerting_channel_management_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.monitoring.v0.CreateAlertingChannelRequest)
  })
_sym_db.RegisterMessage(CreateAlertingChannelRequest)

CreateAlertingChannelResponse = _reflection.GeneratedProtocolMessageType('CreateAlertingChannelResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEALERTINGCHANNELRESPONSE,
  '__module__' : 'qwak.monitoring.v0.alerting_channel_management_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.monitoring.v0.CreateAlertingChannelResponse)
  })
_sym_db.RegisterMessage(CreateAlertingChannelResponse)

UpdateAlertingChannelRequest = _reflection.GeneratedProtocolMessageType('UpdateAlertingChannelRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEALERTINGCHANNELREQUEST,
  '__module__' : 'qwak.monitoring.v0.alerting_channel_management_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.monitoring.v0.UpdateAlertingChannelRequest)
  })
_sym_db.RegisterMessage(UpdateAlertingChannelRequest)

UpdateAlertingChannelResponse = _reflection.GeneratedProtocolMessageType('UpdateAlertingChannelResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEALERTINGCHANNELRESPONSE,
  '__module__' : 'qwak.monitoring.v0.alerting_channel_management_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.monitoring.v0.UpdateAlertingChannelResponse)
  })
_sym_db.RegisterMessage(UpdateAlertingChannelResponse)

GetAlertingChannelRequestById = _reflection.GeneratedProtocolMessageType('GetAlertingChannelRequestById', (_message.Message,), {
  'DESCRIPTOR' : _GETALERTINGCHANNELREQUESTBYID,
  '__module__' : 'qwak.monitoring.v0.alerting_channel_management_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.monitoring.v0.GetAlertingChannelRequestById)
  })
_sym_db.RegisterMessage(GetAlertingChannelRequestById)

GetAlertingChannelResponseById = _reflection.GeneratedProtocolMessageType('GetAlertingChannelResponseById', (_message.Message,), {
  'DESCRIPTOR' : _GETALERTINGCHANNELRESPONSEBYID,
  '__module__' : 'qwak.monitoring.v0.alerting_channel_management_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.monitoring.v0.GetAlertingChannelResponseById)
  })
_sym_db.RegisterMessage(GetAlertingChannelResponseById)

GetAlertingChannelRequestByName = _reflection.GeneratedProtocolMessageType('GetAlertingChannelRequestByName', (_message.Message,), {
  'DESCRIPTOR' : _GETALERTINGCHANNELREQUESTBYNAME,
  '__module__' : 'qwak.monitoring.v0.alerting_channel_management_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.monitoring.v0.GetAlertingChannelRequestByName)
  })
_sym_db.RegisterMessage(GetAlertingChannelRequestByName)

GetAlertingChannelResponseByName = _reflection.GeneratedProtocolMessageType('GetAlertingChannelResponseByName', (_message.Message,), {
  'DESCRIPTOR' : _GETALERTINGCHANNELRESPONSEBYNAME,
  '__module__' : 'qwak.monitoring.v0.alerting_channel_management_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.monitoring.v0.GetAlertingChannelResponseByName)
  })
_sym_db.RegisterMessage(GetAlertingChannelResponseByName)

ListAlertingChannelRequest = _reflection.GeneratedProtocolMessageType('ListAlertingChannelRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTALERTINGCHANNELREQUEST,
  '__module__' : 'qwak.monitoring.v0.alerting_channel_management_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.monitoring.v0.ListAlertingChannelRequest)
  })
_sym_db.RegisterMessage(ListAlertingChannelRequest)

ListAlertingChannelResponse = _reflection.GeneratedProtocolMessageType('ListAlertingChannelResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTALERTINGCHANNELRESPONSE,
  '__module__' : 'qwak.monitoring.v0.alerting_channel_management_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.monitoring.v0.ListAlertingChannelResponse)
  })
_sym_db.RegisterMessage(ListAlertingChannelResponse)

DeleteAlertingChannelRequest = _reflection.GeneratedProtocolMessageType('DeleteAlertingChannelRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEALERTINGCHANNELREQUEST,
  '__module__' : 'qwak.monitoring.v0.alerting_channel_management_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.monitoring.v0.DeleteAlertingChannelRequest)
  })
_sym_db.RegisterMessage(DeleteAlertingChannelRequest)

DeleteAlertingChannelResponse = _reflection.GeneratedProtocolMessageType('DeleteAlertingChannelResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEALERTINGCHANNELRESPONSE,
  '__module__' : 'qwak.monitoring.v0.alerting_channel_management_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.monitoring.v0.DeleteAlertingChannelResponse)
  })
_sym_db.RegisterMessage(DeleteAlertingChannelResponse)

_ALERTINGCHANNELMANAGEMENTSERVICE = DESCRIPTOR.services_by_name['AlertingChannelManagementService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\035com.qwak.ai.monitoring.api.v0P\001Z\017.;monitoring_v0'
  _CREATEALERTINGCHANNELREQUEST._serialized_start=127
  _CREATEALERTINGCHANNELREQUEST._serialized_end=224
  _CREATEALERTINGCHANNELRESPONSE._serialized_start=226
  _CREATEALERTINGCHANNELRESPONSE._serialized_end=257
  _UPDATEALERTINGCHANNELREQUEST._serialized_start=259
  _UPDATEALERTINGCHANNELREQUEST._serialized_end=356
  _UPDATEALERTINGCHANNELRESPONSE._serialized_start=358
  _UPDATEALERTINGCHANNELRESPONSE._serialized_end=389
  _GETALERTINGCHANNELREQUESTBYID._serialized_start=391
  _GETALERTINGCHANNELREQUESTBYID._serialized_end=434
  _GETALERTINGCHANNELRESPONSEBYID._serialized_start=436
  _GETALERTINGCHANNELRESPONSEBYID._serialized_end=537
  _GETALERTINGCHANNELREQUESTBYNAME._serialized_start=539
  _GETALERTINGCHANNELREQUESTBYNAME._serialized_end=586
  _GETALERTINGCHANNELRESPONSEBYNAME._serialized_start=588
  _GETALERTINGCHANNELRESPONSEBYNAME._serialized_end=691
  _LISTALERTINGCHANNELREQUEST._serialized_start=693
  _LISTALERTINGCHANNELREQUEST._serialized_end=721
  _LISTALERTINGCHANNELRESPONSE._serialized_start=723
  _LISTALERTINGCHANNELRESPONSE._serialized_end=821
  _DELETEALERTINGCHANNELREQUEST._serialized_start=823
  _DELETEALERTINGCHANNELREQUEST._serialized_end=867
  _DELETEALERTINGCHANNELRESPONSE._serialized_start=869
  _DELETEALERTINGCHANNELRESPONSE._serialized_end=900
  _ALERTINGCHANNELMANAGEMENTSERVICE._serialized_start=903
  _ALERTINGCHANNELMANAGEMENTSERVICE._serialized_end=1700
# @@protoc_insertion_point(module_scope)



class QwakGeneralBuildException(Exception):
    def __init__(self, message: str, src_exception: Exception = None):
        self._message = message
        self._exception_msg = str(src_exception) if src_exception else ""

    @property
    def message(self) -> str:
        msg = f"""Message: {self._message}
Exception message: {self._exception_msg}"""
        return msg

    def __str__(self):
        return self.message



import grpc
from qwak.inner.tool.auth import Auth0ClientBase

_SIGNATURE_HEADER_KEY = "authorization"


class Auth0Client(grpc.AuthMetadataPlugin, Auth0ClientBase):
    def __init__(self):
        Auth0ClientBase.__init__(self)

    def __call__(self, context, callback):
        callback(((_SIGNATURE_HEADER_KEY, "Bearer {}".format(self.get_token())),), None)



#
#    Copyright 2024 OpenAI
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from dataclasses import dataclass
from typing import List, Optional

__all__ = ["ChatCompletionTokenLogprob", "TopLogprob"]


@dataclass
class TopLogprob:
    token: str
    """The token."""

    logprob: float
    """The log probability of this token, if it is within the top 20 most likely
    tokens.

    Otherwise, the value `-9999.0` is used to signify that the token is very
    unlikely.
    """

    bytes: Optional[List[int]] = None
    """A list of integers representing the UTF-8 bytes representation of the token.

    Useful in instances where characters are represented by multiple tokens and
    their byte representations must be combined to generate the correct text
    representation. Can be `null` if there is no bytes representation for the token.
    """


@dataclass
class ChatCompletionTokenLogprob:
    token: str
    """The token."""

    logprob: float
    """The log probability of this token, if it is within the top 20 most likely
    tokens.

    Otherwise, the value `-9999.0` is used to signify that the token is very
    unlikely.
    """

    top_logprobs: List[TopLogprob]
    """List of the most likely tokens and their log probability, at this token
    position.

    In rare cases, there may be fewer than the number of requested `top_logprobs`
    returned.
    """

    bytes: Optional[List[int]] = None
    """A list of integers representing the UTF-8 bytes representation of the token.

    Useful in instances where characters are represented by multiple tokens and
    their byte representations must be combined to generate the correct text
    representation. Can be `null` if there is no bytes representation for the token.
    """



from typing import List

from _qwak_proto.qwak.audience.v1.audience_api_pb2 import (
    CreateAudienceRequest,
    CreateAudienceResponse,
    DeleteAudienceRequest,
    GetAudienceRequest,
    GetAudienceResponse,
    ListAudienceRequest,
    ListAudienceResponse,
    UpdateAudienceRequest,
)
from _qwak_proto.qwak.audience.v1.audience_api_pb2_grpc import AudienceAPIStub
from _qwak_proto.qwak.audience.v1.audience_pb2 import Audience, AudienceEntry
from dependency_injector.wiring import Provide
from qwak.inner.di_configuration import QwakContainer


class AudienceClient:
    def __init__(self, grpc_channel=Provide[QwakContainer.core_grpc_channel]):
        self._service_stub = AudienceAPIStub(grpc_channel)

    def create_audience(self, audience: Audience) -> str:
        create_audience_request = CreateAudienceRequest(audience=audience)
        create_audience_response: CreateAudienceResponse = (
            self._service_stub.CreateAudience(create_audience_request)
        )
        return create_audience_response.audience_id

    def get_audience(self, audience_id: str) -> Audience:
        get_audience_request = GetAudienceRequest(audience_id=audience_id)
        get_audience_response: GetAudienceResponse = self._service_stub.GetAudience(
            get_audience_request
        )
        return get_audience_response.audience

    def delete_audience(self, audience_id: str) -> None:
        delete_audience_request = DeleteAudienceRequest(audience_id=audience_id)
        self._service_stub.DeleteAudience(delete_audience_request)

    def list_audience(self) -> List[AudienceEntry]:
        list_audience_request = ListAudienceRequest()
        list_audience_response: ListAudienceResponse = self._service_stub.ListAudience(
            list_audience_request
        )
        return list_audience_response.audience_entries

    def update_audience(self, audience_id: str, audience: Audience) -> None:
        update_audience_request = UpdateAudienceRequest(
            id=audience_id, audience=audience
        )
        self._service_stub.UpdateAudience(update_audience_request)



import grpc
from dependency_injector.wiring import Provide

from _qwak_proto.qwak.builds.builds_pb2 import (
    BuildStatus,
    ExperimentTrackingValues,
    GetBuildRequest,
    GetBuildResponse,
    ListBuildsRequest,
    ListBuildsResponse,
    Metric,
    Param,
    RegisterExperimentTrackingValuesRequest,
    RegisterModelSchemaRequest,
    UpdateBuildStatusRequest,
)
from _qwak_proto.qwak.builds.builds_pb2_grpc import BuildsManagementServiceStub
from qwak.exceptions import QwakException
from qwak.inner.di_configuration import QwakContainer


class BuildsManagementClient:
    """
    Used for interacting with Feature Registry endpoints
    """

    def __init__(self, grpc_channel=Provide[QwakContainer.core_grpc_channel]):
        self._builds_management_service = BuildsManagementServiceStub(grpc_channel)

    def update_build_status(self, build_id, build_status, qwak_user_id=""):
        try:
            self._builds_management_service.UpdateBuildStatus(
                UpdateBuildStatusRequest(
                    build_id=build_id,
                    build_status=BuildStatus.Value(build_status),
                    qwak_calling_user_id=qwak_user_id,
                )
            )

        except grpc.RpcError as e:
            raise QwakException(
                f"Failed to update build status, error is {e.details()}"
            )

    def get_build(self, build_id) -> GetBuildResponse:
        try:
            return self._builds_management_service.GetBuild(
                GetBuildRequest(build_id=build_id)
            )
        except grpc.RpcError as e:
            raise QwakException(f"Failed to get build, error is {e.details()}")

    def is_build_exists(self, build_id: str) -> bool:
        try:
            self._builds_management_service.GetBuild(GetBuildRequest(build_id=build_id))
            return True
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.NOT_FOUND:
                return False
            raise QwakException(
                f"Failed to check if build {build_id} is exists, error is {e.details()}"
            )

    def register_schema(self, build_id, model_schema):
        try:
            return self._builds_management_service.RegisterModelSchema(
                RegisterModelSchemaRequest(build_id=build_id, model_schema=model_schema)
            )

        except grpc.RpcError as e:
            raise QwakException(f"Failed to register schema, error is {e.details()}")

    def register_experiment_tracking_values(
        self, build_id, qwak_params=None, cli_params=None, qwak_metrics=None
    ):
        qwak_params = qwak_params if qwak_params else {}
        cli_params = cli_params if cli_params else {}
        qwak_metrics = qwak_metrics if qwak_metrics else {}

        try:
            params = cli_params.copy()
            params.update(qwak_params)

            experiment_tracking_values = ExperimentTrackingValues(
                metrics=[
                    Metric(key=key, value=value) for key, value in qwak_metrics.items()
                ],
                params=[
                    Param(key=key, value=str(value)) for key, value in params.items()
                ],
            )

            return self._builds_management_service.RegisterExperimentTrackingValues(
                RegisterExperimentTrackingValuesRequest(
                    build_id=build_id,
                    experiment_tracking_values=experiment_tracking_values,
                )
            )

        except grpc.RpcError as e:
            raise QwakException(
                f"Failed to register experiment tracking values, error is {e.details()}"
            )

    def list_builds(self, model_uuid="", **kwargs) -> ListBuildsResponse:
        try:
            _model_uuid = model_uuid if model_uuid else kwargs.get("branch_id")
            if not _model_uuid:
                raise QwakException("missing argument model uuid or branch id.")
            return self._builds_management_service.ListBuilds(
                ListBuildsRequest(model_uuid=_model_uuid)
            )
        except grpc.RpcError as e:
            raise QwakException(f"Failed to list builds, error is {e.details()}")



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import qwak.monitoring.v0.alerting_channel_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class CreateAlertingChannelRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> qwak.monitoring.v0.alerting_channel_pb2.AlertingChannelCreateOptions:
        """Parameters for creating an alerting channel"""
    def __init__(
        self,
        *,
        options: qwak.monitoring.v0.alerting_channel_pb2.AlertingChannelCreateOptions | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___CreateAlertingChannelRequest = CreateAlertingChannelRequest

class CreateAlertingChannelResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___CreateAlertingChannelResponse = CreateAlertingChannelResponse

class UpdateAlertingChannelRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> qwak.monitoring.v0.alerting_channel_pb2.AlertingChannelUpdateOptions:
        """Parameters for updating an alerting channel"""
    def __init__(
        self,
        *,
        options: qwak.monitoring.v0.alerting_channel_pb2.AlertingChannelUpdateOptions | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___UpdateAlertingChannelRequest = UpdateAlertingChannelRequest

class UpdateAlertingChannelResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UpdateAlertingChannelResponse = UpdateAlertingChannelResponse

class GetAlertingChannelRequestById(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    id: builtins.str
    """Id of the required alerting channel"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id", b"id"]) -> None: ...

global___GetAlertingChannelRequestById = GetAlertingChannelRequestById

class GetAlertingChannelResponseById(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DESCRIPTION_FIELD_NUMBER: builtins.int
    @property
    def description(self) -> qwak.monitoring.v0.alerting_channel_pb2.AlertingChannelDescription:
        """Alerting channel for the given id"""
    def __init__(
        self,
        *,
        description: qwak.monitoring.v0.alerting_channel_pb2.AlertingChannelDescription | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["description", b"description"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["description", b"description"]) -> None: ...

global___GetAlertingChannelResponseById = GetAlertingChannelResponseById

class GetAlertingChannelRequestByName(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of the required alerting channel"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name"]) -> None: ...

global___GetAlertingChannelRequestByName = GetAlertingChannelRequestByName

class GetAlertingChannelResponseByName(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DESCRIPTION_FIELD_NUMBER: builtins.int
    @property
    def description(self) -> qwak.monitoring.v0.alerting_channel_pb2.AlertingChannelDescription:
        """Alerting channel for the given id"""
    def __init__(
        self,
        *,
        description: qwak.monitoring.v0.alerting_channel_pb2.AlertingChannelDescription | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["description", b"description"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["description", b"description"]) -> None: ...

global___GetAlertingChannelResponseByName = GetAlertingChannelResponseByName

class ListAlertingChannelRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ListAlertingChannelRequest = ListAlertingChannelRequest

class ListAlertingChannelResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DESCRIPTION_FIELD_NUMBER: builtins.int
    @property
    def description(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.monitoring.v0.alerting_channel_pb2.AlertingChannelDescription]:
        """List of found alerting channels"""
    def __init__(
        self,
        *,
        description: collections.abc.Iterable[qwak.monitoring.v0.alerting_channel_pb2.AlertingChannelDescription] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["description", b"description"]) -> None: ...

global___ListAlertingChannelResponse = ListAlertingChannelResponse

class DeleteAlertingChannelRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of the required alerting channel"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name"]) -> None: ...

global___DeleteAlertingChannelRequest = DeleteAlertingChannelRequest

class DeleteAlertingChannelResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DeleteAlertingChannelResponse = DeleteAlertingChannelResponse
